#include "mainwindow.h"

#include <QtGui>

#include <iostream>
#include <typeinfo>

#include "aboutdialog.h"
#include "branchpropwindow.h"
#include "branchitem.h"
#include "exportoofiledialog.h"
#include "exports.h"
#include "file.h"
#include "findresultwidget.h"
#include "findwidget.h"
#include "flagrow.h"
#include "historywindow.h"
#include "imports.h"
#include "mapeditor.h"
#include "misc.h"
#include "options.h"
#include "process.h"
#include "settings.h"
#include "shortcuts.h"
#include "texteditor.h"
#include "warningdialog.h"
#include "xlinkitem.h"

//#include <modeltest.h>	// FIXME-3

#if defined(Q_OS_WIN32)
// Define only this structure as opposed to
// including full 'windows.h'. FindWindow
// clashes with the one in Win32 API.
typedef struct _PROCESS_INFORMATION
{
	long hProcess;
	long hThread;
	long dwProcessId;
	long dwThreadId;
} PROCESS_INFORMATION, *LPPROCESS_INFORMATION;
#endif

extern TextEditor *textEditor;
extern Main *mainWindow;
extern FindWidget *findWidget;
extern FindResultWidget *findResultWidget;
extern QString tmpVymDir;
extern QString clipboardDir;
extern QString clipboardFile;
extern bool clipboardEmpty;
extern int statusbarTime;
extern FlagRow *standardFlagsMaster;	
extern FlagRow *systemFlagsMaster;
extern QString vymName;
extern QString vymVersion;
extern QString vymBuildDate;
extern bool debug;

QMenu* branchContextMenu;
QMenu* branchAddContextMenu;
QMenu* branchRemoveContextMenu;
QMenu* branchLinksContextMenu;
QMenu* branchXLinksContextMenuEdit;
QMenu* floatimageContextMenu;
QMenu* canvasContextMenu;
QMenu* fileLastMapsMenu;
QMenu* fileImportMenu;
QMenu* fileExportMenu;


extern Settings settings;
extern Options options;
extern ImageIO imageIO;

extern QDir vymBaseDir;
extern QDir lastImageDir;
extern QDir lastFileDir;
#if defined(Q_OS_WIN32)
extern QDir vymInstallDir;
#endif
extern QString iconPath;
extern QString flagsPath;


Main::Main(QWidget* parent, const char* name, Qt::WFlags f) :
QMainWindow(parent,name,f)
{
	mainWindow=this;

	setCaption ("VYM - View Your Mind");

	// Load window settings
	#if defined(Q_OS_WIN32)
	if (settings.value("/mainwindow/geometry/maximized", false).toBool())
	{
		setWindowState(Qt::WindowMaximized);
	}
	else
	#endif
	{
		resize (settings.value("/mainwindow/geometry/size", QSize (800,600)).toSize());
		move   (settings.value("/mainwindow/geometry/pos",  QPoint(300,100)).toPoint());
	}

	// Sometimes we may need to remember old selections
	prevSelection="";

	// Default color
	currentColor=Qt::black;

	// Create unique temporary directory
	bool ok;
	tmpVymDir=makeTmpDir (ok,"vym");
	if (!ok)
	{
		qWarning ("Mainwindow: Could not create temporary directory, failed to start vym");
		exit (1);
	}
	if (debug) qDebug (QString("vym tmpDir=%1").arg(tmpVymDir) );

	// Create direcctory for clipboard
	clipboardDir=tmpVymDir+"/clipboard";
	clipboardFile="map.xml";
	QDir d(clipboardDir);
	d.mkdir (clipboardDir,true);
	makeSubDirs (clipboardDir);
	clipboardEmpty=true;

	// Remember PID of our friendly webbrowser
	browserPID=new qint64;
	*browserPID=0;

	// Dock widgets 
	findResultWidget=new FindResultWidget ();
	QDockWidget *dw= new QDockWidget (tr ("Search results","FindResultWidget"),this);
	dw->setWidget (findResultWidget);
	dw->setObjectName ("FindResultWidget");
	dw->hide();	
	addDockWidget (Qt::RightDockWidgetArea,dw);

	// Satellite windows //////////////////////////////////////////
	// history window
	historyWindow=new HistoryWindow();
	connect (historyWindow, SIGNAL (windowClosed() ), this, SLOT (updateActions()));

	// properties window
	branchPropertyWindow = new BranchPropertyWindow();
	connect (branchPropertyWindow, SIGNAL (windowClosed() ), this, SLOT (updateActions()));

	// Connect TextEditor, so that we can update flags if text changes
	connect (textEditor, SIGNAL (textHasChanged() ), this, SLOT (updateNoteFlag()));
	connect (textEditor, SIGNAL (windowClosed() ), this, SLOT (updateActions()));

	// Initialize script editor
	scriptEditor = new SimpleScriptEditor();
	scriptEditor->move (50,50);

	connect( scriptEditor, SIGNAL( runScript ( QString ) ), 
		this, SLOT( runScript( QString ) ) );

	// Initialize some settings, which are platform dependant
	QString p,s;

		// application to open URLs
		p="/mainwindow/readerURL";
		#if defined(Q_OS_LINUX)
			s=settings.value (p,"xdg-open").toString();
		#else
			#if defined(Q_OS_MACX)
				s=settings.value (p,"/usr/bin/open").toString();

			#else
				#if defined(Q_OS_WIN32)
					// Assume that system has been set up so that
					// Explorer automagically opens up the URL
					// in the user's preferred browser.
					s=settings.value (p,"explorer").toString();
				#else
					s=settings.value (p,"mozilla").toString();
				#endif
			#endif
		#endif
		settings.setValue( p,s);

		// application to open PDFs
		p="/mainwindow/readerPDF";
		#if defined(Q_OS_LINUX)
			s=settings.value (p,"xdg-open").toString();
		#else
			#if defined(Q_OS_MACX)
				s=settings.value (p,"/usr/bin/open").toString();
			#elif defined(Q_OS_WIN32)
				s=settings.value (p,"acrord32").toString();
			#else
				s=settings.value (p,"acroread").toString();
			#endif
		#endif
		settings.setValue( p,s);

	// width of xLinksMenu
	xLinkMenuWidth=60;

	// Create Layout
	QWidget* centralWidget = new QWidget (this);
	QVBoxLayout *layout=new QVBoxLayout (centralWidget);
	setCentralWidget(centralWidget);	

	// Create tab widget which holds the maps
	tabWidget= new QTabWidget (centralWidget);
	connect( tabWidget, SIGNAL( currentChanged( QWidget * ) ), 
		this, SLOT( editorChanged( QWidget * ) ) );

	// Create findWidget
	findWidget = new FindWidget (centralWidget);
	findWidget->hide();
	layout->addWidget (tabWidget);
	layout->addWidget (findWidget);

	connect (
		findWidget, SIGNAL (nextButton (QString) ), 
		this, SLOT (editFindNext(QString) ) );
    connect (
        findWidget , SIGNAL (hideFindWidget() ),
        this, SLOT (editHideFindWidget() ) );

	setupFileActions();
	setupEditActions();
	setupFormatActions();
	setupViewActions();
	setupModeActions();
	setupFlagActions();
	setupNetworkActions();
	setupSettingsActions();
	setupContextMenus();
	setupMacros();
	if (debug) switchboard.print();

	if (settings.value( "/mainwindow/showTestMenu",false).toBool()) setupTestActions();
	setupHelpActions();

	// Status bar and progress bar there
	statusBar();
	progressMax=0;
	progressCounter=0;
	progressCounterTotal=0;

	progressDialog.setLabelText (tr("Loading maps","Mainwindow"));
	progressDialog.setAutoReset(false);
	progressDialog.setAutoClose(false);
	//progressDialog.setWindowModality (Qt::WindowModal);	// That forces mainwindo to update and slows down
	//progressDialog.setCancelButton (NULL);

	restoreState (settings.value("/mainwindow/state",0).toByteArray());

	updateGeometry();
}

Main::~Main()
{
	// Save Settings
	#if defined(Q_OS_WIN32)
	settings.setValue ("/mainwindow/geometry/maximized", isMaximized());
	#endif
	settings.setValue ("/mainwindow/geometry/size", size());
	settings.setValue ("/mainwindow/geometry/pos", pos());
	settings.setValue ("/mainwindow/state",saveState(0));

	settings.setValue ("/mainwindow/view/AntiAlias",actionViewToggleAntiAlias->isOn());
	settings.setValue ("/mainwindow/view/SmoothPixmapTransform",actionViewToggleSmoothPixmapTransform->isOn());
	settings.setValue( "/version/version", vymVersion );
	settings.setValue( "/version/builddate", vymBuildDate );

	settings.setValue( "/mainwindow/autosave/use",actionSettingsAutosaveToggle->isOn() );
	settings.setValue ("/mainwindow/autoLayout/use",actionSettingsAutoLayoutToggle->isOn() );
	settings.setValue( "/mapeditor/editmode/autoSelectNewBranch",actionSettingsAutoSelectNewBranch->isOn() );
	settings.setValue( "/mainwindow/writeBackupFile",actionSettingsWriteBackupFile->isOn() );
	settings.setValue( "/mapeditor/editmode/autoSelectText",actionSettingsAutoSelectText->isOn() );
	settings.setValue( "/mapeditor/editmode/autoEditNewBranch",actionSettingsAutoEditNewBranch->isOn() );
	settings.setValue( "/mapeditor/editmode/useDelKey",actionSettingsUseDelKey->isOn() );
	settings.setValue( "/mapeditor/editmode/useFlagGroups",actionSettingsUseFlagGroups->isOn() );
	settings.setValue( "/export/useHideExport",actionSettingsUseHideExport->isOn() );

	//TODO save scriptEditor settings

	// call the destructors
	delete textEditor;
	delete historyWindow;
	delete branchPropertyWindow;

	// Remove temporary directory
	removeDir (QDir(tmpVymDir));
}

void Main::loadCmdLine()
{
	/* TODO draw some kind of splashscreen while loading...
	if (qApp->argc()>1)
	{
	}
	*/

	QStringList flist=options.getFileList();
	QStringList::Iterator it=flist.begin();

	progressCounter=flist.count();
	progressCounterTotal=flist.count();
	while (it !=flist.end() )
	{
		fileLoad (*it, NewMap);
		*it++;
	}	
}


void Main::statusMessage(const QString &s)
{
	// Surpress messages while progressdialog during 
	// load is active
	if (progressMax==0)
		statusBar()->message( s);
}

void Main::setProgressMaximum (int max)	
{
	if (progressCounterTotal!=0)
	
		progressDialog.setRange (0,progressCounterTotal*1000);
	else
		progressDialog.setRange (0,max+10);

	progressDialog.setValue (0);
	progressMax=max*1000;
	//cout << "Main  max="<<max<<"  v="<<progressDialog.value()<<endl;
	progressDialog.show();
}

void Main::addProgressValue (float v)
{
	//cout << "addVal v="<<v*1000<<"/"<<progressMax<<"  cur="<<progressDialog.value()<<"  counter="<<v+progressCounter<<"/"<<progressCounterTotal<<endl;
	if (progressCounterTotal!=0)
		progressDialog.setValue ( (v+progressCounterTotal-progressCounter)*1000 );
	else	
		progressDialog.setValue (v+progressDialog.value());
}

void Main::removeProgressCounter()
{
	progressMax=0;
	progressCounter--;
	if (progressCounter<=0)
	{ 
		// Hide dialog again
		progressCounterTotal=0;
		progressDialog.reset();
		progressDialog.hide();
	}	
}

void Main::closeEvent (QCloseEvent* )
{
	fileExitVYM();
}

// File Actions
void Main::setupFileActions()
{
	QMenu *fileMenu = menuBar()->addMenu ( tr ("&Map") );
	QToolBar *tb = addToolBar( tr ("&Map") );
	tb->setObjectName ("mapTB");

	QAction *a;
	a = new QAction(QPixmap( iconPath+"filenew.png"), tr( "&New map","File menu" ),this);
	a->setStatusTip ( tr( "New map","Status tip File menu" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_N );		//New map
	switchboard.addConnection(a,tr("File","Shortcut group"));
	a->addTo( tb );
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileNew() ) );

	a = new QAction(QPixmap( iconPath+"filenewcopy.png"), tr( "&Copy to new map","File menu" ),this);
	a->setStatusTip ( tr( "Copy selection to mapcenter of a new map","Status tip File menu" ) );
	a->setShortcut ( Qt::CTRL +Qt::SHIFT + Qt::Key_N );		//New map
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileNewCopy() ) );
	actionFileNewCopy=a;

	a = new QAction( QPixmap( iconPath+"fileopen.png"), tr( "&Open..." ,"File menu"),this);
	a->setStatusTip (tr( "Open","Status tip File menu" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_O );		//Open map
	switchboard.addConnection(a,tr("File","Shortcut group"));
	a->addTo( tb );
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileLoad() ) );

	fileLastMapsMenu = fileMenu->addMenu (tr("Open Recent","File menu"));
	fileMenu->addSeparator();

	a = new QAction( QPixmap( iconPath+"filesave.png"), tr( "&Save...","File menu" ), this);
	a->setStatusTip ( tr( "Save","Status tip file menu" ));
	a->setShortcut (Qt::CTRL + Qt::Key_S );			//Save map
	switchboard.addConnection(a,tr("File","Shortcut group"));
	a->addTo( tb );
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileSave() ) );
	actionFileSave=a;

	a = new QAction( QPixmap(iconPath+"filesaveas.png"), tr( "Save &As...","File menu" ), this);
	a->setStatusTip (tr( "Save &As","Status tip file menu" ) );
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileSaveAs() ) );

	fileMenu->addSeparator();

	fileImportMenu = fileMenu->addMenu (tr("Import","File menu"));

	a = new QAction(tr("KDE 3 Bookmarks"), this);
	a->setStatusTip ( tr( "Import %1","Status tip file menu" ).arg(tr("KDE 3 bookmarks")));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	a->addTo (fileImportMenu);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileImportKDE3Bookmarks() ) );

	a = new QAction(tr("KDE 4 Bookmarks"), this);
	a->setStatusTip ( tr( "Import %1","Status tip file menu" ).arg(tr("KDE 4 bookmarks")));
	a->addTo (fileImportMenu);
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileImportKDE4Bookmarks() ) );

	if (settings.value( "/mainwindow/showTestMenu",false).toBool()) 
	{
		a = new QAction( QPixmap(), tr("Firefox Bookmarks","File menu"),this);
		a->setStatusTip (tr( "Import %1","Status tip file menu").arg(tr("Firefox Bookmarks" ) ));
		a->addTo (fileImportMenu);
		switchboard.addConnection(a,tr("File","Shortcut group"));
		connect( a, SIGNAL( triggered() ), this, SLOT( fileImportFirefoxBookmarks() ) );
	}	

	a = new QAction("Freemind...",this);
	a->setStatusTip ( tr( "Import %1","status tip file menu").arg(" Freemind")  );
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileImportMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileImportFreemind() ) );

	a = new QAction("Mind Manager...",this);
	a->setStatusTip ( tr( "Import %1","status tip file menu").arg(" Mind Manager")  );
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileImportMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileImportMM() ) );

	a = new QAction( tr( "Import Dir%1","File menu").arg("..."), this);
	a->setStatusTip (tr( "Import directory structure (experimental)","status tip file menu" ) );
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileImportMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileImportDir() ) );

	fileExportMenu = fileMenu->addMenu (tr("Export","File menu"));

	a = new QAction( tr("Image%1","File export menu").arg("..."), this);
	a->setStatusTip( tr( "Export map as image","status tip file menu" ));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportImage() ) );
	fileExportMenu->addAction (a);

	a = new QAction( "Open Office...", this);
	a->setStatusTip( tr( "Export in Open Document Format used e.g. in Open Office ","status tip file menu" ));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportOOPresentation() ) );
	fileExportMenu->addAction (a);

	a = new QAction(  "Webpage (HTML)...",this );
	a->setShortcut (Qt::ALT + Qt::Key_X);			//Export HTML
	a->setStatusTip ( tr( "Export as %1","status tip file menu").arg(tr(" webpage (XHTML)","status tip file menu")));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportHTML() ) );
	fileExportMenu->addAction (a);

	a = new QAction(  "Webpage (XHTML)...",this );	//Export XHTML

	//a->setShortcut (Qt::ALT + Qt::SHIFT + Qt::Key_X);	a->setStatusTip ( tr( "Export as %1","status tip file menu").arg(tr(" webpage (XHTML)","status tip file menu")));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportXHTML() ) );
	fileExportMenu->addAction (a);

	a = new QAction( "Text (A&O report)...", this);
	a->setStatusTip ( tr( "Export as %1").arg("A&O report "+tr("(still experimental)" )));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportAO() ) );
	fileExportMenu->addAction (a);

	a = new QAction( "Text (ASCII)...", this);
	a->setStatusTip ( tr( "Export as %1").arg("ASCII "+tr("(still experimental)" )));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportASCII() ) );
	fileExportMenu->addAction (a);

	a = new QAction( "Spreadsheet (CSV)...", this);
	a->setStatusTip ( tr( "Export as %1").arg("CSV "+tr("(still experimental)" )));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportCSV() ) );
	fileExportMenu->addAction (a);

	a = new QAction( tr("KDE 3 Bookmarks","File menu"), this);
	a->setStatusTip( tr( "Export as %1").arg(tr("KDE 3 Bookmarks" )));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportKDE3Bookmarks() ) );
	fileExportMenu->addAction (a);

	a = new QAction( tr("KDE 4 Bookmarks","File menu"), this);
	a->setStatusTip( tr( "Export as %1").arg(tr("KDE 4 Bookmarks" )));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportKDE4Bookmarks() ) );
	fileExportMenu->addAction (a);

	a = new QAction( "Taskjuggler...", this );
	a->setStatusTip( tr( "Export as %1").arg("Taskjuggler "+tr("(still experimental)" )));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportTaskjuggler() ) );
	fileExportMenu->addAction (a);

	a = new QAction( "LaTeX...", this);
	a->setStatusTip( tr( "Export as %1").arg("LaTeX "+tr("(still experimental)" )));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportLaTeX() ) );
	fileExportMenu->addAction (a);

	a = new QAction( "XML..." , this );
	a->setStatusTip (tr( "Export as %1").arg("XML"));
	switchboard.addConnection(a,tr("File","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExportXML() ) );
	fileExportMenu->addAction (a);

	fileMenu->addSeparator();

	a = new QAction(QPixmap( iconPath+"fileprint.png"), tr( "&Print")+QString("..."), this);
	a->setStatusTip ( tr( "Print" ,"File menu") );
	a->setShortcut (Qt::CTRL + Qt::Key_P );			//Print map
	a->addTo( tb );
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( filePrint() ) );
	actionFilePrint=a;

	a = new QAction( QPixmap(iconPath+"fileclose.png"), tr( "&Close Map","File menu" ), this);
	a->setStatusTip (tr( "Close Map" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_W );			//Close map
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileCloseMap() ) );

	a = new QAction(QPixmap(iconPath+"exit.png"), tr( "E&xit","File menu")+" "+vymName, this);
	a->setStatusTip ( tr( "Exit")+" "+vymName );
	a->setShortcut (Qt::CTRL + Qt::Key_Q );			//Quit vym
	switchboard.addConnection(a,tr("File","Shortcut group"));
	fileMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( fileExitVYM() ) );
}


//Edit Actions
void Main::setupEditActions()
{
	QToolBar *tb = addToolBar( tr ("&Actions toolbar","Toolbar name") );
	tb->setLabel( "Edit Actions" );
	tb->setObjectName ("actionsTB");
	QMenu *editMenu = menuBar()->addMenu( tr("&Edit","Edit menu") );

	QAction *a;
	a = new QAction( QPixmap( iconPath+"undo.png"), tr( "&Undo","Edit menu" ),this);
	connect( a, SIGNAL( triggered() ), this, SLOT( editUndo() ) );
	a->setStatusTip (tr( "Undo" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_Z );		//Undo last action
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	tb->addAction (a);
	editMenu->addAction (a);
	actionUndo=a;

	a = new QAction( QPixmap( iconPath+"redo.png"), tr( "&Redo","Edit menu" ), this); 
	a->setStatusTip (tr( "Redo" ));
	a->setShortcut (Qt::CTRL + Qt::Key_Y );			//Redo last action
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	tb->addAction (a);
	editMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editRedo() ) );
	actionRedo=a;

	editMenu->addSeparator();
	a = new QAction(QPixmap( iconPath+"editcopy.png"), tr( "&Copy","Edit menu" ), this);
	a->setStatusTip ( tr( "Copy" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_C );			//Copy
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	tb->addAction (a);
	editMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editCopy() ) );
	actionCopy=a;

	a = new QAction(QPixmap( iconPath+"editcut.png" ), tr( "Cu&t","Edit menu" ), this);
	a->setStatusTip ( tr( "Cut" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_X );			//Cut
	a->setEnabled (false);
	tb->addAction (a);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	editMenu->addAction (a);
	actionCut=a;
	connect( a, SIGNAL( triggered() ), this, SLOT( editCut() ) );

	a = new QAction(QPixmap( iconPath+"editpaste.png"), tr( "&Paste","Edit menu" ),this);
	connect( a, SIGNAL( triggered() ), this, SLOT( editPaste() ) );
	a->setStatusTip ( tr( "Paste" ) );
	a->setShortcut ( Qt::CTRL + Qt::Key_V );		//Paste
	a->setEnabled (false);
	tb->addAction (a);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	editMenu->addAction (a);
	actionPaste=a;

	// Shortcut to delete selection
	a = new QAction( tr( "Delete Selection","Edit menu" ),this);
	a->setStatusTip (tr( "Delete Selection" ));
	a->setShortcut ( Qt::Key_Delete);				//Delete selection
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editDeleteSelection() ) );
	actionDelete=a;

	// Shortcut to add attribute
	a= new QAction(tr( "Add attribute" ), this);
	a->setShortcut ( Qt::Key_Q);	
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editAddAttribute() ) );
	actionAddAttribute= a;


	// Shortcut to add mapcenter
	a= new QAction(QPixmap(iconPath+"newmapcenter.png"),tr( "Add mapcenter","Canvas context menu" ), this);
	a->setShortcut ( Qt::Key_M);	
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editAddMapCenter() ) );
	//actionListBranches.append(a);
	tb->addAction (a);
	actionAddMapCenter = a;


	// Shortcut to add branch
	a = new QAction(QPixmap(iconPath+"newbranch.png"), tr( "Add branch as child","Edit menu" ), this);
	a->setStatusTip ( tr( "Add a branch as child of selection" ));
	a->setShortcut (Qt::Key_A);					//Add branch
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editNewBranch() ) );
	a = new QAction(QPixmap(iconPath+"newbranch.png"), tr( "Add branch as child","Edit menu" ), this);
	a->setStatusTip ( tr( "Add a branch as child of selection" ));
	a->setShortcut (Qt::Key_Insert);				//Add branch
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editNewBranch() ) );
	actionListBranches.append(a);
	actionAddBranch=a;
	editMenu->addAction (actionAddBranch);
	tb->addAction (actionAddBranch);


	// Add branch by inserting it at selection
	a = new QAction(tr( "Add branch (insert)","Edit menu" ),this);
	a->setStatusTip ( tr( "Add a branch by inserting and making selection its child" ));
	a->setShortcut ( Qt::ALT + Qt::Key_A );			//Insert branch
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editNewBranchBefore() ) );
	actionListBranches.append(a);
	actionAddBranchBefore=a;

	// Add branch above
	a = new QAction(tr( "Add branch above","Edit menu" ), this);
	a->setStatusTip ( tr( "Add a branch above selection" ));
	a->setShortcut (Qt::SHIFT+Qt::Key_Insert );		//Add branch above
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editNewBranchAbove() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionAddBranchAbove=a;
	a = new QAction(tr( "Add branch above","Edit menu" ), this);
	a->setStatusTip ( tr( "Add a branch above selection" ));
	a->setShortcut (Qt::SHIFT+Qt::Key_A );			//Add branch above
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editNewBranchAbove() ) );
	actionListBranches.append(a);

	// Add branch below 
	a = new QAction(tr( "Add branch below","Edit menu" ), this);
	a->setStatusTip ( tr( "Add a branch below selection" ));
	a->setShortcut (Qt::CTRL +Qt::Key_Insert );		//Add branch below
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editNewBranchBelow() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionAddBranchBelow=a;
	a = new QAction(tr( "Add branch below","Edit menu" ), this);
	a->setStatusTip ( tr( "Add a branch below selection" ));
	a->setShortcut (Qt::CTRL +Qt::Key_A );			// Add branch below
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editNewBranchBelow() ) );
	actionListBranches.append(a);

	a = new QAction(QPixmap(iconPath+"up.png" ), tr( "Move up","Edit menu" ), this);
	a->setStatusTip ( tr( "Move branch up" ) );
	a->setShortcut (Qt::Key_PageUp );				// Move branch up
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	tb->addAction (a);
	editMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editMoveUp() ) );
	actionMoveUp=a;

	a = new QAction( QPixmap( iconPath+"down.png"), tr( "Move down","Edit menu" ),this);
	connect( a, SIGNAL( triggered() ), this, SLOT( editMoveDown() ) );
	a->setStatusTip (tr( "Move branch down" ) );
	a->setShortcut ( Qt::Key_PageDown );			// Move branch down
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	tb->addAction (a);
	editMenu->addAction (a);
	actionMoveDown=a;

	a = new QAction(QPixmap(), tr( "&Detach","Context menu" ),this);
	a->setStatusTip ( tr( "Detach branch and use as mapcenter","Context menu" ) );
	a->setShortcut ( Qt::Key_D );					// Detach branch
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	editMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editDetach() ) );
	actionDetach=a;

	a = new QAction( QPixmap(iconPath+"editsort.png" ), tr( "Sort children","Edit menu" ), this );
	connect( a, SIGNAL( activated() ), this, SLOT( editSortChildren() ) );
	a->setEnabled (true);
	a->addTo( tb );
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	editMenu->addAction (a);
	actionSortChildren=a;

	a = new QAction( QPixmap(iconPath+"editsortback.png" ), tr( "Sort children backwards","Edit menu" ), this );
	connect( a, SIGNAL( activated() ), this, SLOT( editSortBackChildren() ) );
	a->setEnabled (true);
	a->addTo( tb );
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	editMenu->addAction (a);
	actionSortBackChildren=a;	

	a = new QAction( QPixmap(flagsPath+"flag-scrolled-right.png"), tr( "Scroll branch","Edit menu" ), this);
	a->setShortcut ( Qt::Key_S );					// Scroll branch
	a->setStatusTip (tr( "Scroll branch" )); 
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editToggleScroll() ) );
	a->setEnabled (false);
	a->setToggleAction(true);
	actionToggleScroll=a;
	tb->addAction (actionToggleScroll);
	editMenu->addAction ( actionToggleScroll);
	editMenu->addAction (actionToggleScroll);
	addAction (a);
	actionListBranches.append(actionToggleScroll);

	a = new QAction( QPixmap(), tr( "Expand all branches","Edit menu" ), this);
	a->setShortcut ( Qt::SHIFT + Qt::Key_X );		// Expand all branches 
	a->setStatusTip (tr( "Expand all branches" )); 
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editExpandAll() ) );
	actionExpandAll=a;
	actionExpandAll->setEnabled (false);
	actionExpandAll->setToggleAction(false);
	//tb->addAction (actionExpandAll);
	editMenu->addAction ( actionExpandAll);
	addAction (a);
	actionListBranches.append(actionExpandAll);

	a = new QAction( QPixmap(), tr( "Expand one level","Edit menu" ), this);
	a->setShortcut ( Qt::Key_Greater );		// Expand one level in tree editor
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	a->setStatusTip (tr( "Expand one level in tree editor" )); 
	connect( a, SIGNAL( triggered() ), this, SLOT( editExpandOneLevel() ) );
	a->setEnabled (false);
	a->setToggleAction(false);
	actionExpandOneLevel=a;
	//tb->addAction (a);
	editMenu->addAction ( a);
	addAction (a);
	actionListBranches.append(a);

	a = new QAction( QPixmap(), tr( "Collapse one level","Edit menu" ), this);
	a->setShortcut ( Qt::Key_Less);			// Collapse one level in tree editor
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	a->setStatusTip (tr( "Collapse one level in tree editor" )); 
	connect( a, SIGNAL( triggered() ), this, SLOT( editCollapseOneLevel() ) );
	a->setEnabled (false);
	a->setToggleAction(false);
	actionCollapseOneLevel=a;
	//tb->addAction (a);
	editMenu->addAction ( a);
	addAction (a);
	actionListBranches.append(a);

	a = new QAction( tr( "Unscroll children","Edit menu" ), this);
	a->setStatusTip (tr( "Unscroll all scrolled branches in selected subtree" ));
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	editMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editUnscrollChildren() ) );

	editMenu->addSeparator();

	a = new QAction( QPixmap(iconPath+"find.png"), tr( "Find...","Edit menu"), this);
	a->setStatusTip (tr( "Find" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_F );				//Find
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	editMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editOpenFindWidget() ) );

	a = new QAction( tr( "Find duplicate URLs","Edit menu"), this);
	//a->setStatusTip (tr( "Find" ) );
	a->setShortcut (Qt::SHIFT + Qt::Key_F);				//Find duplicate URLs
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	if (settings.value( "/mainwindow/showTestMenu",false).toBool() ) 
		editMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editFindDuplicateURLs() ) );

	editMenu->addSeparator();

	a = new QAction( QPixmap(flagsPath+"flag-url.png"), tr( "Open URL","Edit menu" ), this);
	a->setShortcut (Qt::SHIFT + Qt::Key_U );
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	tb->addAction (a);
	addAction(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editOpenURL() ) );
	actionOpenURL=a;

	a = new QAction( tr( "Open URL in new tab","Edit menu" ), this);
	a->setStatusTip (tr( "Open URL in new tab" ));
	//a->setShortcut (Qt::CTRL+Qt::Key_U );
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editOpenURLTab() ) );
	actionOpenURLTab=a;

	a = new QAction( tr( "Open all URLs in subtree (including scrolled branches)","Edit menu" ), this);
	a->setStatusTip (tr( "Open all URLs in subtree (including scrolled branches)" ));
	a->setShortcut ( Qt::CTRL + Qt::Key_U );
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction(a);
	actionListBranches.append(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editOpenMultipleVisURLTabs() ) );
	actionOpenMultipleVisURLTabs=a;

	a = new QAction( tr( "Open all URLs in subtree","Edit menu" ), this);
	a->setStatusTip (tr( "Open all URLs in subtree" ));
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction(a);
	actionListBranches.append(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editOpenMultipleURLTabs() ) );
	actionOpenMultipleURLTabs=a;

	a = new QAction(QPixmap(), tr( "Edit URL...","Edit menu"), this);
	a->setStatusTip ( tr( "Edit URL" ) );
	a->setShortcut ( Qt::Key_U );
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	actionListBranches.append(a);
	addAction(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editURL() ) );
	actionURL=a;

	a = new QAction(QPixmap(), tr( "Edit local URL...","Edit menu"), this);
	a->setStatusTip ( tr( "Edit local URL" ) );
	//a->setShortcut (Qt::SHIFT +  Qt::Key_U );
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	actionListBranches.append(a);
	addAction(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editLocalURL() ) );
	actionLocalURL=a;

	a = new QAction( tr( "Use heading for URL","Edit menu" ), this);
	a->setStatusTip ( tr( "Use heading of selected branch as URL" ));
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	actionListBranches.append(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editHeading2URL() ) );
	actionHeading2URL=a;

	a = new QAction(tr( "Create URL to Novell Bugzilla","Edit menu" ), this);
	a->setStatusTip ( tr( "Create URL to Novell Bugzilla" ));
	a->setEnabled (false);
	actionListBranches.append(a);
	a->setShortcut ( Qt::Key_B );
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editBugzilla2URL() ) );
	actionBugzilla2URL=a;

	a = new QAction(tr( "Get data from Novell Bugzilla","Edit menu" ), this);
	a->setStatusTip ( tr( "Get data from Novell Bugzilla" ));
	a->setEnabled (false);
	actionListBranches.append(a);
	a->setShortcut ( Qt::Key_B + Qt::SHIFT);
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( getBugzillaData() ) );
	actionGetBugzillaData=a;

	a = new QAction(tr( "Create URL to Novell FATE","Edit menu" ), this);
	a->setStatusTip ( tr( "Create URL to Novell FATE" ));
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	actionListBranches.append(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editFATE2URL() ) );
	actionFATE2URL=a;

	a = new QAction(QPixmap(flagsPath+"flag-vymlink.png"), tr( "Open linked map","Edit menu" ), this);
	a->setStatusTip ( tr( "Jump to another vym map, if needed load it first" ));
	tb->addAction (a);
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editOpenVymLink() ) );
	actionOpenVymLink=a;

	a = new QAction(QPixmap(), tr( "Open all vym links in subtree","Edit menu" ), this);
	a->setStatusTip ( tr( "Open all vym links in subtree" ));
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	actionListBranches.append(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editOpenMultipleVymLinks() ) );
	actionOpenMultipleVymLinks=a;


	a = new QAction(tr( "Edit vym link...","Edit menu" ), this);
	a->setEnabled (false);
	a->setStatusTip ( tr( "Edit link to another vym map" ));
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editVymLink() ) );
	actionListBranches.append(a);
	actionVymLink=a;

	a = new QAction(tr( "Delete vym link","Edit menu" ),this);
	a->setStatusTip ( tr( "Delete link to another vym map" ));
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editDeleteVymLink() ) );
	actionDeleteVymLink=a;

	a = new QAction(QPixmap(flagsPath+"flag-hideexport.png"), tr( "Hide in exports","Edit menu" ), this);
	a->setStatusTip ( tr( "Hide object in exports" ) );
	a->setShortcut (Qt::Key_H );
	a->setToggleAction(true);
	tb->addAction (a);
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editToggleHideExport() ) );
	actionToggleHideExport=a;

	a = new QAction(tr( "Add timestamp","Edit menu" ), this);
	a->setStatusTip ( tr( "Add timestamp" ));
	a->setEnabled (false);
	actionListBranches.append(a);
	a->setShortcut ( Qt::Key_T );	// Add timestamp
	a->setShortcutContext (Qt::WindowShortcut);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction(a);
	connect( a, SIGNAL( triggered() ), this, SLOT( editAddTimestamp() ) );
	actionAddTimestamp=a;

	a = new QAction(tr( "Edit Map Info...","Edit menu" ),this);
	a->setStatusTip ( tr( "Edit Map Info" ));
	a->setEnabled (true);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editMapInfo() ) );
	actionMapInfo=a;

	// Import at selection (adding to selection)
	a = new QAction( tr( "Add map (insert)","Edit menu" ),this);
	a->setStatusTip (tr( "Add map at selection" ));
	connect( a, SIGNAL( triggered() ), this, SLOT( editImportAdd() ) );
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	actionListBranches.append(a);
	actionImportAdd=a;

	// Import at selection (replacing selection)
	a = new QAction( tr( "Add map (replace)","Edit menu" ), this);
	a->setStatusTip (tr( "Replace selection with map" ));
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editImportReplace() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionImportReplace=a;

	// Save selection 
	a = new QAction( tr( "Save selection","Edit menu" ), this);
	a->setStatusTip (tr( "Save selection" ));
	connect( a, SIGNAL( triggered() ), this, SLOT( editSaveBranch() ) );
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	actionListBranches.append(a);
	actionSaveBranch=a;

	// Only remove branch, not its children
	a = new QAction(tr( "Remove only branch ","Edit menu" ), this);
	a->setStatusTip ( tr( "Remove only branch and keep its children" ));
	a->setShortcut (Qt::ALT + Qt::Key_Delete );
	connect( a, SIGNAL( triggered() ), this, SLOT( editDeleteKeepChildren() ) );
	a->setEnabled (false);
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	addAction (a);
	actionListBranches.append(a);
	actionDeleteKeepChildren=a;

	// Only remove children of a branch
	a = new QAction( tr( "Remove children","Edit menu" ), this);
	a->setStatusTip (tr( "Remove children of branch" ));
	a->setShortcut (Qt::SHIFT + Qt::Key_Delete );
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editDeleteChildren() ) );
	a->setEnabled (false);
	actionListBranches.append(a);
	actionDeleteChildren=a;

	a = new QAction( tr( "Add Image...","Edit menu" ), this);
	a->setStatusTip (tr( "Add Image" ));
	switchboard.addConnection(a,tr("Edit","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( editLoadImage() ) );
	actionLoadImage=a;

	a = new QAction( tr( "Property window","Dialog to edit properties of selection" )+QString ("..."), this);
	a->setStatusTip (tr( "Set properties for selection" ));
	a->setShortcut ( Qt::CTRL + Qt::Key_I );		//Property window
	a->setShortcutContext (Qt::WindowShortcut);
	a->setToggleAction (true);
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( windowToggleProperty() ) );
	actionViewTogglePropertyWindow=a;
}

// Format Actions
void Main::setupFormatActions()
{
	QMenu *formatMenu = menuBar()->addMenu (tr ("F&ormat","Format menu"));

	QToolBar *tb = addToolBar( tr("Format Actions","Format Toolbar name"));
	tb->setObjectName ("formatTB");
	QAction *a;
	QPixmap pix( 16,16);
	pix.fill (Qt::black);
	a= new QAction(pix, tr( "Set &Color" )+QString("..."), this);
	a->setStatusTip ( tr( "Set Color" ));
	connect( a, SIGNAL( triggered() ), this, SLOT( formatSelectColor() ) );
	a->addTo( tb );
	formatMenu->addAction (a);
	actionFormatColor=a;
	a= new QAction( QPixmap(iconPath+"formatcolorpicker.png"), tr( "Pic&k color","Edit menu" ), this);
	a->setStatusTip (tr( "Pick color\nHint: You can pick a color from another branch and color using CTRL+Left Button" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_K );
	connect( a, SIGNAL( triggered() ), this, SLOT( formatPickColor() ) );
	a->setEnabled (false);
	a->addTo( tb );
	formatMenu->addAction (a);
	actionListBranches.append(a);
	actionFormatPickColor=a;

	a= new QAction(QPixmap(iconPath+"formatcolorbranch.png"), tr( "Color &branch","Edit menu" ), this);
	a->setStatusTip ( tr( "Color branch" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_B + Qt::SHIFT);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatColorBranch() ) );
	a->setEnabled (false);
	a->addTo( tb );
	formatMenu->addAction (a);
	actionListBranches.append(a);
	actionFormatColorSubtree=a;

	a= new QAction(QPixmap(iconPath+"formatcolorsubtree.png"), tr( "Color sub&tree","Edit menu" ), this);
	a->setStatusTip ( tr( "Color Subtree" ));
	a->setShortcut (Qt::CTRL + Qt::Key_B);	// Color subtree
	connect( a, SIGNAL( triggered() ), this, SLOT( formatColorSubtree() ) );
	a->setEnabled (false);
	formatMenu->addAction (a);
	a->addTo( tb );
	actionListBranches.append(a);
	actionFormatColorSubtree=a;

	formatMenu->addSeparator();
	actionGroupFormatLinkStyles=new QActionGroup ( this);
	actionGroupFormatLinkStyles->setExclusive (true);
	a= new QAction( tr( "Linkstyle Line" ), actionGroupFormatLinkStyles);
	a->setStatusTip (tr( "Line" ));
	a->setToggleAction(true);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatLinkStyleLine() ) );
	formatMenu->addAction (a);
	actionFormatLinkStyleLine=a;
	a= new QAction( tr( "Linkstyle Curve" ), actionGroupFormatLinkStyles);
	a->setStatusTip (tr( "Line" ));
	a->setToggleAction(true);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatLinkStyleParabel() ) );
	formatMenu->addAction (a);
	actionFormatLinkStyleParabel=a;
	a= new QAction( tr( "Linkstyle Thick Line" ), actionGroupFormatLinkStyles );
	a->setStatusTip (tr( "PolyLine" ));
	a->setToggleAction(true);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatLinkStylePolyLine() ) );
	formatMenu->addAction (a);
	actionFormatLinkStylePolyLine=a;
	a= new QAction( tr( "Linkstyle Thick Curve" ), actionGroupFormatLinkStyles);
	a->setStatusTip (tr( "PolyParabel" ) );
	a->setToggleAction(true);
	a->setChecked (true);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatLinkStylePolyParabel() ) );
	formatMenu->addAction (a);
	actionFormatLinkStylePolyParabel=a;

	a = new QAction( tr( "Hide link if object is not selected","Branch attribute" ), this);
	a->setStatusTip (tr( "Hide link" ));
	a->setToggleAction(true);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatHideLinkUnselected() ) );
	actionFormatHideLinkUnselected=a;

	formatMenu->addSeparator();
	a= new QAction( tr( "&Use color of heading for link","Branch attribute" ),  this);
	a->setStatusTip (tr( "Use same color for links and headings" ));
	a->setToggleAction(true);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatToggleLinkColorHint() ) );
	formatMenu->addAction (a);
	actionFormatLinkColorHint=a;

	pix.fill (Qt::white);
	a= new QAction( pix, tr( "Set &Link Color"+QString("...") ), this  );
	a->setStatusTip (tr( "Set Link Color" ));
	formatMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatSelectLinkColor() ) );
	actionFormatLinkColor=a;

	a= new QAction( pix, tr( "Set &Selection Color"+QString("...") ), this  );
	a->setStatusTip (tr( "Set Selection Color" ));
	formatMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatSelectSelectionColor() ) );
	actionFormatSelectionColor=a;

	a= new QAction( pix, tr( "Set &Background Color" )+QString("..."), this );
	a->setStatusTip (tr( "Set Background Color" ));
	formatMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatSelectBackColor() ) );
	actionFormatBackColor=a;

	a= new QAction( pix, tr( "Set &Background image" )+QString("..."), this );
	a->setStatusTip (tr( "Set Background image" ));
	formatMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( formatSelectBackImage() ) );
	actionFormatBackImage=a;
}

// View Actions
void Main::setupViewActions()
{
	QToolBar *tb = addToolBar( tr("View Actions","View Toolbar name") );
	tb->setLabel( "View Actions" );
	tb->setObjectName ("viewTB");
	QMenu *viewMenu = menuBar()->addMenu ( tr( "&View" ));


	QAction *a;
	a = new QAction(QPixmap(iconPath+"viewmag-reset.png"), tr( "reset Zoom","View action" ), this);
	a->setStatusTip ( tr( "Zoom reset" ) );
	a->setShortcut (Qt::CTRL + Qt::Key_0);	// Reset zoom
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	a->addTo( tb );
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(viewZoomReset() ) );

	a = new QAction( QPixmap(iconPath+"viewmag+.png"), tr( "Zoom in","View action" ), this);
	a->setStatusTip (tr( "Zoom in" ));
	a->setShortcut(Qt::CTRL + Qt::Key_Plus);
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	a->addTo( tb );
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(viewZoomIn() ) );

	a = new QAction( QPixmap(iconPath+"viewmag-.png"), tr( "Zoom out","View action" ), this);
	a->setStatusTip (tr( "Zoom out" ));
	a->setShortcut(Qt::CTRL + Qt::Key_Minus);
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	a->addTo( tb );
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( viewZoomOut() ) );

	a = new QAction( QPixmap(iconPath+"viewshowsel.png"), tr( "Show selection","View action" ), this);
	a->setStatusTip (tr( "Show selection" ));
	a->setShortcut(Qt::Key_Period);
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	a->addTo( tb );
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( viewCenter() ) );

	viewMenu->addSeparator();	

	a = new QAction(QPixmap(flagsPath+"flag-note.png"), tr( "Show Note Editor","View action" ),this);
	a->setStatusTip ( tr( "Show Note Editor" ));
	a->setShortcut ( Qt::CTRL + Qt::Key_E );	// Toggle Note Editor
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	a->setToggleAction(true);
	a->addTo( tb );
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(windowToggleNoteEditor() ) );
	actionViewToggleNoteEditor=a;

	a = new QAction(QPixmap(), tr( "Show tree editor","View action" ),this);
	a->setStatusTip ( tr( "Show tree editor" ));
	a->setShortcut ( Qt::CTRL + Qt::Key_T );	// Toggle Tree Editor // FIXME-3 originally: color subtree
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	a->setToggleAction(true);
	a->addTo( tb );
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(windowToggleTreeEditor() ) );
	actionViewToggleTreeEditor=a;

	a = new QAction(QPixmap(iconPath+"history.png"),  tr( "History Window","View action" ),this );
	a->setStatusTip ( tr( "Show History Window" ));
	a->setShortcut ( Qt::CTRL + Qt::Key_H  );	// Toggle history window
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	a->setToggleAction(true);
	a->addTo( tb );
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(windowToggleHistory() ) );
	actionViewToggleHistoryWindow=a;

	viewMenu->addAction (actionViewTogglePropertyWindow);

	viewMenu->addSeparator();	

	a = new QAction(tr( "Antialiasing","View action" ),this );
	a->setStatusTip ( tr( "Antialiasing" ));
	a->setToggleAction(true);
	a->setChecked (settings.value("/mainwindow/view/AntiAlias",true).toBool());
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(windowToggleAntiAlias() ) );
	actionViewToggleAntiAlias=a;

	a = new QAction(tr( "Smooth pixmap transformations","View action" ),this );
	a->setStatusTip (a->text());
	a->setToggleAction(true);
	a->setChecked (settings.value("/mainwindow/view/SmoothPixmapTransformation",true).toBool());
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(windowToggleSmoothPixmap() ) );
	actionViewToggleSmoothPixmapTransform=a;

	a = new QAction(tr( "Next Map","View action" ), this);
	a->setStatusTip (a->text());
	a->setShortcut (Qt::ALT + Qt::Key_N );
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(windowNextEditor() ) );

	a = new QAction (tr( "Previous Map","View action" ), this );
	a->setStatusTip (a->text());
	a->setShortcut (Qt::ALT + Qt::Key_P );
	switchboard.addConnection(a,tr("View shortcuts","Shortcut group"));
	viewMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT(windowPreviousEditor() ) );
}

// Mode Actions
void Main::setupModeActions()
{
	//QPopupMenu *menu = new QPopupMenu( this );
	//menuBar()->insertItem( tr( "&Mode (using modifiers)" ), menu );

	QToolBar *tb = addToolBar( tr ("Modes when using modifiers","Modifier Toolbar name") );
	tb->setObjectName ("modesTB");
	QAction *a;
	actionGroupModModes=new QActionGroup ( this);
	actionGroupModModes->setExclusive (true);
	a= new QAction( QPixmap(iconPath+"modecolor.png"), tr( "Use modifier to color branches","Mode modifier" ), actionGroupModModes);
	a->setShortcut (Qt::Key_J);
	switchboard.addConnection(a,tr("Modes","Shortcut group"));
	a->setStatusTip ( tr( "Use modifier to color branches" ));
	a->setToggleAction(true);
	a->addTo (tb);
	a->setChecked(true);
	actionModModeColor=a;

	a= new QAction( QPixmap(iconPath+"modecopy.png"), tr( "Use modifier to copy","Mode modifier" ), actionGroupModModes );
	a->setShortcut( Qt::Key_K); 
	switchboard.addConnection(a,tr("Modes","Shortcut group"));
	a->setStatusTip( tr( "Use modifier to copy" ));
	a->setToggleAction(true);
	a->addTo (tb);
	actionModModeCopy=a;

	a= new QAction(QPixmap(iconPath+"modelink.png"), tr( "Use modifier to draw xLinks","Mode modifier" ), actionGroupModModes );
	a->setShortcut (Qt::Key_L);
	switchboard.addConnection(a,tr("Modes","Shortcut group"));
	a->setStatusTip( tr( "Use modifier to draw xLinks" ));
	a->setToggleAction(true);
	a->addTo (tb);
	actionModModeXLink=a;
}

// Flag Actions
void Main::setupFlagActions()
{
	// Create System Flags
	QToolBar *tb=NULL;

	Flag *flag=new Flag(flagsPath+"flag-note.png");
	setupFlag (flag,tb,"system-note",tr("Note","SystemFlag"));

	flag=new Flag(flagsPath+"flag-url.png");
	setupFlag (flag,tb,"system-url",tr("URL to Document ","SystemFlag"));

	flag=new Flag(flagsPath+"flag-url-bugzilla-novell.png");
	setupFlag (flag,tb,"system-url-bugzilla-novell",tr("URL to Bugzilla ","SystemFlag"));

	flag=new Flag(flagsPath+"flag-vymlink.png");
	setupFlag (flag,tb,"system-vymLink",tr("Link to another vym map","SystemFlag"));

	flag=new Flag(flagsPath+"flag-scrolled-right.png");
	setupFlag (flag,tb,"system-scrolledright",tr("subtree is scrolled","SystemFlag"));

	flag=new Flag(flagsPath+"flag-tmpUnscrolled-right.png");
	setupFlag (flag,tb,"system-tmpUnscrolledRight",tr("subtree is temporary scrolled","SystemFlag"));

	flag=new Flag(flagsPath+"flag-hideexport.png");
	setupFlag (flag,tb,"system-hideInExport",tr("Hide object in exported maps","SystemFlag"));

	// Create Standard Flags
	tb=addToolBar (tr ("Standard Flags","Standard Flag Toolbar"));
	tb->setObjectName ("standardFlagTB");
	standardFlagsMaster->setToolBar (tb);

	flag=new Flag(flagsPath+"flag-exclamationmark.png");
	flag->setGroup("standard-mark");
	setupFlag (flag,tb,"exclamationmark",tr("Take care!","Standardflag"));

	flag=new Flag(flagsPath+"flag-questionmark.png");
	flag->setGroup("standard-mark");
	setupFlag (flag,tb,"questionmark",tr("Really?","Standardflag"));

	flag=new Flag(flagsPath+"flag-hook-green.png");
	flag->setGroup("standard-status");
	setupFlag (flag,tb,"hook-green",tr("Status - ok,done","Standardflag"));

	flag=new Flag(flagsPath+"flag-wip.png");
	flag->setGroup("standard-status");
	setupFlag (flag,tb,"wip",tr("Status - work in progress","Standardflag"));

	flag=new Flag(flagsPath+"flag-cross-red.png");
	flag->setGroup("standard-status");
	setupFlag (flag,tb,"cross-red",tr("Status - missing, not started","Standardflag"));
	flag->unsetGroup();

	flag=new Flag(flagsPath+"flag-stopsign.png");
	setupFlag (flag,tb,"stopsign",tr("This won't work!","Standardflag"));

	flag=new Flag(flagsPath+"flag-smiley-good.png");
	flag->setGroup("standard-smiley");
	setupFlag (flag,tb,"smiley-good",tr("Good","Standardflag"));

	flag=new Flag(flagsPath+"flag-smiley-sad.png");
	flag->setGroup("standard-smiley");
	setupFlag (flag,tb,"smiley-sad",tr("Bad","Standardflag"));

	flag=new Flag(flagsPath+"flag-smiley-omg.png");
	flag->setGroup("standard-smiley");
	setupFlag (flag,tb,"smiley-omb",tr("Oh no!","Standardflag"));
	// Original omg.png (in KDE emoticons)
	flag->unsetGroup();

	flag=new Flag(flagsPath+"flag-kalarm.png");
	setupFlag (flag,tb,"clock",tr("Time critical","Standardflag"));

	flag=new Flag(flagsPath+"flag-phone.png");
	setupFlag (flag,tb,"phone",tr("Call...","Standardflag"));

	flag=new Flag(flagsPath+"flag-lamp.png");
	setupFlag (flag,tb,"lamp",tr("Idea!","Standardflag"));

	flag=new Flag(flagsPath+"flag-arrow-up.png");
	flag->setGroup("standard-arrow");
	setupFlag (flag,tb,"arrow-up",tr("Important","Standardflag"));

	flag=new Flag(flagsPath+"flag-arrow-down.png");
	flag->setGroup("standard-arrow");
	setupFlag (flag,tb,"arrow-down",tr("Unimportant","Standardflag"));

	flag=new Flag(flagsPath+"flag-arrow-2up.png");
	flag->setGroup("standard-arrow");
	setupFlag (flag,tb,"2arrow-up",tr("Very important!","Standardflag"));

	flag=new Flag(flagsPath+"flag-arrow-2down.png");
	flag->setGroup("standard-arrow");
	setupFlag (flag,tb,"2arrow-down",tr("Very unimportant!","Standardflag"));
	flag->unsetGroup();

	flag=new Flag(flagsPath+"flag-thumb-up.png");
	flag->setGroup("standard-thumb");
	setupFlag (flag,tb,"thumb-up",tr("I like this","Standardflag"));

	flag=new Flag(flagsPath+"flag-thumb-down.png");
	flag->setGroup("standard-thumb");
	setupFlag (flag,tb,"thumb-down",tr("I do not like this","Standardflag"));
	flag->unsetGroup();

	flag=new Flag(flagsPath+"flag-rose.png");
	setupFlag (flag,tb,"rose",tr("Rose","Standardflag"));

	flag=new Flag(flagsPath+"flag-heart.png");
	setupFlag (flag,tb,"heart",tr("I just love...","Standardflag"));

	flag=new Flag(flagsPath+"flag-present.png");
	setupFlag (flag,tb,"present",tr("Surprise!","Standardflag"));

	flag=new Flag(flagsPath+"flag-flash.png");
	setupFlag (flag,tb,"flash",tr("Dangerous","Standardflag"));

	// Original: xsldbg_output.png
	flag=new Flag(flagsPath+"flag-info.png");
	setupFlag (flag,tb,"info",tr("Info","Standardflag"));

	// Original khelpcenter.png
	flag=new Flag(flagsPath+"flag-lifebelt.png");
	setupFlag (flag,tb,"lifebelt",tr("This will help","Standardflag"));

	// Freemind flags
	flag=new Flag(flagsPath+"freemind/warning.png");
	flag->setVisible(false);
	setupFlag (flag,tb,  "freemind-warning",tr("Important","Freemind-Flag"));

	for (int i=1; i<8; i++)
	{
		flag=new Flag(flagsPath+QString("freemind/priority-%1.png").arg(i));
		setupFlag (flag,tb, QString("freemind-priority-%1").arg(i),tr("Priority","Freemind-Flag"));
	}

	flag=new Flag(flagsPath+"freemind/back.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-back",tr("Back","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/forward.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-forward",tr("forward","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/attach.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-attach",tr("Look here","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/clanbomber.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-clanbomber",tr("Dangerous","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/desktopnew.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-desktopnew",tr("Don't flagrget","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/flag.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-flag",tr("Flag","Freemind-Flag"));


	flag=new Flag(flagsPath+"freemind/gohome.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-gohome",tr("Home","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/kaddressbook.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-kaddressbook",tr("Telephone","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/knotify.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-knotify",tr("Music","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/korn.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-korn",tr("Mailbox","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/mail.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-mail",tr("Maix","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/password.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-password",tr("Password","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/pencil.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-pencil",tr("To be improved","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/stop.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-stop",tr("Stop","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/wizard.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-wizard",tr("Magic","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/xmag.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-xmag",tr("To be discussed","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/bell.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-bell",tr("Reminder","Freemind-Flag"));

	flag=new Flag(flagsPath+"freemind/bookmark.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-bookmark",tr("Excellent","Freemind-Flag"));

	flag= new Flag(flagsPath+"freemind/penguin.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-penguin",tr("Linux","Freemind-Flag"));

	flag=new Flag (flagsPath+"freemind/licq.png");
	flag->setVisible(false);
	setupFlag (flag,tb,"freemind-licq",tr("Sweet","Freemind-Flag"));
}

void Main::setupFlag (Flag *flag, QToolBar *tb, const QString &name, const QString &tooltip)
{
	flag->setName(name);
	flag->setToolTip (tooltip);
	QAction *a;
	if (tb)
	{
		a=new QAction (flag->getPixmap(),name,this);
		// StandardFlag
		tb->addAction (a);
		flag->setAction (a);
		a->setVisible (flag->isVisible());
		a->setCheckable(true);
		a->setObjectName(name);
		a->setToolTip(tooltip);
		connect (a, SIGNAL( triggered() ), this, SLOT( standardFlagChanged() ) );
		standardFlagsMaster->addFlag (flag);	
	} else
	{
		// SystemFlag
		systemFlagsMaster->addFlag (flag);	
	}
}

// Network Actions
void Main::setupNetworkActions()
{
	if (!settings.value( "/mainwindow/showTestMenu",false).toBool() ) 
		return;
	QMenu *netMenu = menuBar()->addMenu(  "Network" );

	QAction *a;

	a = new QAction(  "Start TCPserver for MapEditor",this);
	//a->setStatusTip ( "Set application to open pdf files"));
	//a->setShortcut ( Qt::ALT + Qt::Key_T );		//New TCP server
	switchboard.addConnection(a,tr("Network shortcuts","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( networkStartServer() ) );
	netMenu->addAction (a);

	a = new QAction(  "Connect MapEditor to server",this);
	//a->setStatusTip ( "Set application to open pdf files"));
	a->setShortcut ( Qt::ALT + Qt::Key_C );		// Connect to server
	switchboard.addConnection(a,tr("Network shortcuts","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( networkConnect() ) );
	netMenu->addAction (a);
}

// Settings Actions
void Main::setupSettingsActions()
{
	QMenu *settingsMenu = menuBar()->addMenu( tr( "&Settings" ));

	QAction *a;

	a = new QAction( tr( "Set application to open pdf files","Settings action"), this);
	a->setStatusTip ( tr( "Set application to open pdf files"));
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsPDF() ) );
	settingsMenu->addAction (a);

	a = new QAction( tr( "Set application to open external links","Settings action"), this);
	a->setStatusTip( tr( "Set application to open external links"));
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsURL() ) );
	settingsMenu->addAction (a);

	a = new QAction( tr( "Set path for macros","Settings action")+"...", this);
	a->setStatusTip( tr( "Set path for macros"));
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsMacroDir() ) );
	settingsMenu->addAction (a);

	a = new QAction( tr( "Set number of undo levels","Settings action")+"...", this);
	a->setStatusTip( tr( "Set number of undo levels"));
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsUndoLevels() ) );
	settingsMenu->addAction (a);

	settingsMenu->addSeparator();

	a = new QAction( tr( "Autosave","Settings action"), this);
	a->setStatusTip( tr( "Autosave"));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mainwindow/autosave/use",false).toBool());
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsAutosaveToggle() ) );
	settingsMenu->addAction (a);
	actionSettingsAutosaveToggle=a;

	a = new QAction( tr( "Autosave time","Settings action")+"...", this);
	a->setStatusTip( tr( "Autosave time"));
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsAutosaveTime() ) );
	settingsMenu->addAction (a);
	actionSettingsAutosaveTime=a;

	a = new QAction( tr( "Write backup file on save","Settings action"), this);
	a->setStatusTip( tr( "Write backup file on save"));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mainwindow/writeBackupFile",false).toBool());
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsWriteBackupFileToggle() ) );
	settingsMenu->addAction (a);
	actionSettingsWriteBackupFile=a;

	settingsMenu->addSeparator();

	a = new QAction( tr( "Edit branch after adding it","Settings action" ), this );
	a->setStatusTip( tr( "Edit branch after adding it" ));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mapeditor/editmode/autoEditNewBranch",true).toBool());
	settingsMenu->addAction (a);
	actionSettingsAutoEditNewBranch=a;

	a= new QAction( tr( "Select branch after adding it","Settings action" ), this );
	a->setStatusTip( tr( "Select branch after adding it" ));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mapeditor/editmode/autoSelectNewBranch",false).toBool() );
	settingsMenu->addAction (a);
	actionSettingsAutoSelectNewBranch=a;

	a= new QAction(tr( "Select existing heading","Settings action" ), this);
	a->setStatusTip( tr( "Select heading before editing" ));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mapeditor/editmode/autoSelectText",true).toBool() );
	settingsMenu->addAction (a);
	actionSettingsAutoSelectText=a;

	a= new QAction( tr( "Delete key","Settings action" ), this);
	a->setStatusTip( tr( "Delete key for deleting branches" ));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mapeditor/editmode/useDelKey",true).toBool() );
	settingsMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsToggleDelKey() ) );
	actionSettingsUseDelKey=a;

	a= new QAction( tr( "Exclusive flags","Settings action" ), this);
	a->setStatusTip( tr( "Use exclusive flags in flag toolbars" ));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mapeditor/editmode/useFlagGroups",true).toBool() );
	settingsMenu->addAction (a);
	actionSettingsUseFlagGroups=a;

	a= new QAction( tr( "Use hide flags","Settings action" ), this);
	a->setStatusTip( tr( "Use hide flag during exports " ));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/export/useHideExport",true).toBool() );
	settingsMenu->addAction (a);
	actionSettingsUseHideExport=a;

	settingsMenu->addSeparator();

	a = new QAction( tr( "Animation","Settings action"), this);
	a->setStatusTip( tr( "Animation"));
	a->setToggleAction(true);
	a->setChecked (settings.value("/animation/use",true).toBool() );
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsToggleAnimation() ) );
	settingsMenu->addAction (a);
	actionSettingsUseAnimation=a;

	a = new QAction( tr( "Automatic layout","Settings action"), this);
	a->setStatusTip( tr( "Automatic layout"));
	a->setToggleAction(true);
	a->setChecked ( settings.value ("/mainwindow/autoLayout/use",true).toBool());
	connect( a, SIGNAL( triggered() ), this, SLOT( settingsAutoLayoutToggle() ) );
	settingsMenu->addAction (a);
	actionSettingsAutoLayoutToggle=a;

}

// Test Actions
void Main::setupTestActions()
{
	QMenu *testMenu = menuBar()->addMenu( tr( "Test" ));

	QAction *a;
	a = new QAction( "Test function 1" , this);
	a->setStatusTip( "Call test function 1" );
	a->setShortcut (Qt::SHIFT + Qt::Key_T);	// Test function 1  
	switchboard.addConnection(a,tr("Test shortcuts","Shortcut group"));
	testMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( testFunction1() ) );

	a = new QAction( "Test function 2" , this);
	a->setStatusTip( "Call test function 2" );
	a->setShortcut (Qt::ALT + Qt::Key_T);	// Test function 2
	switchboard.addConnection(a,tr("Test shortcuts","Shortcut group"));
	testMenu->addAction (a);
	connect( a, SIGNAL( triggered() ), this, SLOT( testFunction2() ) );

	a = new QAction( "Command" , this);
	a->setStatusTip( "Enter command to call in editor" );
	switchboard.addConnection(a,tr("Test shortcuts","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( testCommand() ) );
	testMenu->addAction (a);
}

// Help Actions
void Main::setupHelpActions()
{
	QMenu *helpMenu = menuBar()->addMenu ( tr( "&Help","Help menubar entry" ));

	QAction *a;
	a = new QAction(  tr( "Open VYM Documentation (pdf) ","Help action" ), this );
	a->setStatusTip( tr( "Open VYM Documentation (pdf)" ));
	switchboard.addConnection(a,tr("Help shortcuts","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( helpDoc() ) );
	helpMenu->addAction (a);

	a = new QAction(  tr( "Open VYM example maps ","Help action" ), this );
	a->setStatusTip( tr( "Open VYM example maps " ));
	switchboard.addConnection(a,tr("Help shortcuts","Shortcut group"));
	connect( a, SIGNAL( triggered() ), this, SLOT( helpDemo() ) );
	helpMenu->addAction (a);

	a = new QAction( tr( "About VYM","Help action" ), this);
	a->setStatusTip( tr( "About VYM")+vymName);
	connect( a, SIGNAL( triggered() ), this, SLOT( helpAbout() ) );
	helpMenu->addAction (a);

	a = new QAction( tr( "About QT","Help action" ), this);
	a->setStatusTip( tr( "Information about QT toolkit" ));
	connect( a, SIGNAL( triggered() ), this, SLOT( helpAboutQT() ) );
	helpMenu->addAction (a);
}

// Context Menus
void Main::setupContextMenus()
{
	// Context Menu for branch or mapcenter
	branchContextMenu =new QMenu (this);
	branchContextMenu->addAction (actionViewTogglePropertyWindow);
	branchContextMenu->addSeparator();	

		// Submenu "Add"
		branchAddContextMenu =branchContextMenu->addMenu (tr("Add"));
		branchAddContextMenu->addAction (actionPaste );
		branchAddContextMenu->addAction ( actionAddMapCenter );
		branchAddContextMenu->addAction ( actionAddBranch );
		branchAddContextMenu->addAction ( actionAddBranchBefore );
		branchAddContextMenu->addAction ( actionAddBranchAbove);
		branchAddContextMenu->addAction ( actionAddBranchBelow );
		branchAddContextMenu->addSeparator();	
		branchAddContextMenu->addAction ( actionImportAdd );
		branchAddContextMenu->addAction ( actionImportReplace );

		// Submenu "Remove"
		branchRemoveContextMenu =branchContextMenu->addMenu (tr ("Remove","Context menu name"));
		branchRemoveContextMenu->addAction (actionCut);
		branchRemoveContextMenu->addAction ( actionDelete );
		branchRemoveContextMenu->addAction ( actionDeleteKeepChildren );
		branchRemoveContextMenu->addAction ( actionDeleteChildren );
		

	actionSaveBranch->addTo( branchContextMenu );
	actionFileNewCopy->addTo (branchContextMenu );
	actionDetach->addTo (branchContextMenu );

	branchContextMenu->addSeparator();	
	branchContextMenu->addAction ( actionLoadImage);
	if (settings.value( "/mainwindow/showTestMenu",false).toBool() )
		branchContextMenu->addAction ( actionAddAttribute);

	// Submenu for Links (URLs, vymLinks)
	branchLinksContextMenu =new QMenu (this);

		branchContextMenu->addSeparator();	
		branchLinksContextMenu=branchContextMenu->addMenu(tr("References (URLs, vymLinks, ...)","Context menu name"));	
		branchLinksContextMenu->addAction ( actionOpenURL );
		branchLinksContextMenu->addAction ( actionOpenURLTab );
		branchLinksContextMenu->addAction ( actionOpenMultipleVisURLTabs );
		branchLinksContextMenu->addAction ( actionOpenMultipleURLTabs );
		branchLinksContextMenu->addAction ( actionURL );
		branchLinksContextMenu->addAction ( actionLocalURL );
		branchLinksContextMenu->addAction ( actionHeading2URL );
		branchLinksContextMenu->addAction ( actionBugzilla2URL );
		if (settings.value( "/mainwindow/showTestMenu",false).toBool() )
		{
			branchLinksContextMenu->addAction ( actionGetBugzillaData );
			branchLinksContextMenu->addAction ( actionFATE2URL );
		}	
		branchLinksContextMenu->addSeparator();	
		branchLinksContextMenu->addAction ( actionOpenVymLink );
		branchLinksContextMenu->addAction ( actionOpenMultipleVymLinks );
		branchLinksContextMenu->addAction ( actionVymLink );
		branchLinksContextMenu->addAction ( actionDeleteVymLink );
		

	// Context Menu for XLinks in a branch menu
	// This will be populated "on demand" in MapEditor::updateActions
	branchContextMenu->addSeparator();	
	branchXLinksContextMenuEdit =branchContextMenu->addMenu (tr ("Edit XLink","Context menu name"));
	connect( branchXLinksContextMenuEdit, SIGNAL( triggered(QAction *) ), this, SLOT( editEditXLink(QAction * ) ) );


	// Context menu for floatimage
	floatimageContextMenu =new QMenu (this);
	QAction *a= new QAction (tr ("Save image","Context action"),this);
	connect (a, SIGNAL (triggered()), this, SLOT (editSaveImage()));
	floatimageContextMenu->addAction (a);

	floatimageContextMenu->addSeparator();	
	actionCopy->addTo( floatimageContextMenu );
	actionCut->addTo( floatimageContextMenu );

	floatimageContextMenu->addSeparator();	
	floatimageContextMenu->addAction ( actionFormatHideLinkUnselected );


	// Context menu for canvas
	canvasContextMenu =new QMenu (this);
	actionAddMapCenter->addTo( canvasContextMenu );
	actionMapInfo->addTo( canvasContextMenu );
	canvasContextMenu->insertSeparator();	
	actionGroupFormatLinkStyles->addTo( canvasContextMenu );
	canvasContextMenu->insertSeparator();	
	actionFormatLinkColorHint->addTo( canvasContextMenu );
	actionFormatLinkColor->addTo( canvasContextMenu );
	actionFormatSelectionColor->addTo( canvasContextMenu );
	actionFormatBackColor->addTo( canvasContextMenu );
	// actionFormatBackImage->addTo( canvasContextMenu );  //FIXME-4 makes vym too slow: postponed for later version 

	// Menu for last opened files
	// Create actions
	for (int i = 0; i < MaxRecentFiles; ++i) 
	{
		recentFileActions[i] = new QAction(this);
		recentFileActions[i]->setVisible(false);
		fileLastMapsMenu->addAction(recentFileActions[i]);
		connect(recentFileActions[i], SIGNAL(triggered()),
				this, SLOT(fileLoadRecent()));
	}
	setupRecentMapsMenu();
}

void Main::setupRecentMapsMenu()
{
	QStringList files = settings.value("/mainwindow/recentFileList").toStringList();

	int numRecentFiles = qMin(files.size(), (int)MaxRecentFiles);

	for (int i = 0; i < numRecentFiles; ++i) {
		QString text = tr("&%1 %2").arg(i + 1).arg(files[i]);
		recentFileActions[i]->setText(text);
		recentFileActions[i]->setData(files[i]);
		recentFileActions[i]->setVisible(true);
	}
	for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
		recentFileActions[j]->setVisible(false);
}

void Main::setupMacros()
{
	for (int i = 0; i <= 11; i++) 
	{
		macroActions[i] = new QAction(this);
		macroActions[i]->setData(i);
		addAction (macroActions[i]);
		//switchboard.addConnection(macroActions[i],tr("Macro shortcuts","Shortcut group"));
		connect(macroActions[i], SIGNAL(triggered()),
				this, SLOT(callMacro()));
	}			
	macroActions[0]->setShortcut ( Qt::Key_F1 );
	macroActions[1]->setShortcut ( Qt::Key_F2 );
	macroActions[2]->setShortcut ( Qt::Key_F3 );
	macroActions[3]->setShortcut ( Qt::Key_F4 );
	macroActions[4]->setShortcut ( Qt::Key_F5 );
	macroActions[5]->setShortcut ( Qt::Key_F6 );
	macroActions[6]->setShortcut ( Qt::Key_F7 );
	macroActions[7]->setShortcut ( Qt::Key_F8 );
	macroActions[8]->setShortcut ( Qt::Key_F9 );
	macroActions[9]->setShortcut ( Qt::Key_F10 );
	macroActions[10]->setShortcut ( Qt::Key_F11 );
	macroActions[11]->setShortcut ( Qt::Key_F12 );
}

void Main::hideEvent (QHideEvent * )
{
	if (!textEditor->isMinimized() ) textEditor->hide();
}

void Main::showEvent (QShowEvent * )
{
	if (actionViewToggleNoteEditor->isOn()) textEditor->showNormal();
}


MapEditor* Main::currentMapEditor() const
{
	if ( tabWidget->currentPage())
		return vymViews.at(tabWidget->currentIndex())->getMapEditor();
	return NULL;	
}

uint  Main::currentModelID() const
{
	if (currentModel())
		return currentModel()->getID();
	return 0;	
}

VymModel* Main::currentModel() const
{
	if ( tabWidget->currentPage())
		return vymViews.at(tabWidget->currentIndex())->getModel();
	return NULL;	
}

VymModel* Main::getModel(uint id) const	
{
	// Used in BugAgent
	for (int i=0; i<vymViews.count();i++)
		if (vymViews.at(i)->getModel()->getID()==id)
			return vymViews.at(i)->getModel();
	return NULL;	
}


void Main::editorChanged(QWidget *)
{
	// Unselect all possibly selected objects
	// (Important to update note editor)
	VymModel *m;
	for (int i=0;i<=tabWidget->count() -1;i++)
	{
		m= vymViews.at(i)->getModel();
		if (m) m->unselect();
	}
	m=currentModel();
	if (m) m->reselect();

	// Update actions to in menus and toolbars according to editor
	updateActions();
}

void Main::fileNew()
{
	VymModel *vm=new VymModel;

	/////////////////////////////////////
//	new ModelTest(vm, this);	//FIXME-3
	/////////////////////////////////////

	VymView *vv=new VymView (vm);
	vymViews.append (vv);
	tabWidget->addTab (vv,tr("unnamed","MainWindow: name for new and empty file"));
	tabWidget->setCurrentIndex (vymViews.count() );
	vv->initFocus();

	// Create MapCenter for empty map
	vm->addMapCenter();
	vm->makeDefault();

	// For the very first map we do not have flagrows yet...
	vm->select("mc:");
}

void Main::fileNewCopy()
{
	QString fn="unnamed";
	VymModel *srcModel=currentModel();
	if (srcModel)
	{
		srcModel->copy();
		fileNew();
		VymModel *dstModel=vymViews.last()->getModel();
		dstModel->select("mc:");
		dstModel->load (clipboardDir+"/"+clipboardFile,ImportReplace, VymMap);
	}
}

ErrorCode Main::fileLoad(QString fn, const LoadMode &lmode, const FileType &ftype)
{
	ErrorCode err=success;

	// fn is usually the archive, mapfile the file after uncompressing
	QString mapfile;

	// Make fn absolute (needed for unzip)
	fn=QDir (fn).absPath();

	VymModel *vm;

	if (lmode==NewMap)
	{
		// Check, if map is already loaded
		int i=0;
		while (i<=tabWidget->count() -1)
		{
			if (vymViews.at(i)->getModel()->getFilePath() == fn)
			{
				// Already there, ask for confirmation
				QMessageBox mb( vymName,
					tr("The map %1\nis already opened."
					"Opening the same map in multiple editors may lead \n"
					"to confusion when finishing working with vym."
					"Do you want to").arg(fn),
					QMessageBox::Warning,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::Cancel | QMessageBox::Escape,
					QMessageBox::NoButton);
				mb.setButtonText( QMessageBox::Yes, tr("Open anyway") );
				mb.setButtonText( QMessageBox::Cancel, tr("Cancel"));
				switch( mb.exec() ) 
				{
					case QMessageBox::Yes:
						// end loop and load anyway
						i=tabWidget->count();
						break;
					case QMessageBox::Cancel:
						// do nothing
						return aborted;
						break;
				}
			}
			i++;
		}
	}

	int tabIndex=tabWidget->currentPageIndex();

	// Try to load map
    if ( !fn.isEmpty() )
	{
		vm = currentModel();
		// Check first, if mapeditor exists
		// If it is not default AND we want a new map, 
		// create a new mapeditor in a new tab
		if ( lmode==NewMap && (!vm || !vm->isDefault() )  )
		{
			vm=new VymModel;
			VymView *vv=new VymView (vm);
			vymViews.append (vv);
			tabWidget->addTab (vv,fn);
			tabIndex=tabWidget->count()-1;
			tabWidget->setCurrentPage (tabIndex);
			vv->initFocus();
		}
		
		// Check, if file exists (important for creating new files
		// from command line
		if (!QFile(fn).exists() )
		{
			QMessageBox mb( vymName,
				tr("This map does not exist:\n  %1\nDo you want to create a new one?").arg(fn),
				QMessageBox::Question,
				QMessageBox::Yes ,
				QMessageBox::Cancel | QMessageBox::Default,
				QMessageBox::NoButton );

			mb.setButtonText( QMessageBox::Yes, tr("Create"));
			mb.setButtonText( QMessageBox::No, tr("Cancel"));
			switch( mb.exec() ) 
			{
				case QMessageBox::Yes:
					// Create new map
					currentMapEditor()->getModel()->setFilePath(fn);
					tabWidget->setTabText (tabIndex,
						currentMapEditor()->getModel()->getFileName() );
					statusBar()->message( "Created " + fn , statusbarTime );
					return success;
						
				case QMessageBox::Cancel:
					// don't create new map
					statusBar()->message( "Loading " + fn + " failed!", statusbarTime );
					fileCloseMap();
					return aborted;
			}
		}	


		//tabWidget->currentPage() won't be NULL here, because of above...
		tabWidget->setCurrentIndex (tabIndex);
		//FIXME-3 no me anymore... me->viewport()->setFocus();

		if (err!=aborted)
		{
			// Save existing filename in case  we import
			QString fn_org=vm->getFilePath();

			// Finally load map into mapEditor
			vm->setFilePath (fn);
			err=vm->load(fn,lmode,ftype);

			// Restore old (maybe empty) filepath, if this is an import
			if (lmode!=NewMap)
				vm->setFilePath (fn_org);
		}	

		// Finally check for errors and go home
		if (err==aborted) 
		{
			if (lmode==NewMap) fileCloseMap();
			statusBar()->message( "Could not load " + fn, statusbarTime );
		} else 
		{
			if (lmode==NewMap)
			{
				vm->setFilePath (fn);
				tabWidget->setTabText (tabIndex, vm->getFileName());
				if (!isInTmpDir (fn))
				{
					// Only append to lastMaps if not loaded from a tmpDir
					// e.g. imported bookmarks are in a tmpDir
					addRecentMap(vm->getFilePath() );
				}
				actionFilePrint->setEnabled (true);
			}	
			statusBar()->message( "Loaded " + fn, statusbarTime );
		}	
	}
	return err;
}


void Main::fileLoad(const LoadMode &lmode)
{
	QStringList filters;
	filters <<"VYM map (*.vym *.vyp)"<<"XML (*.xml)";
	QFileDialog *fd=new QFileDialog( this);
	fd->setDir (lastFileDir);
	fd->setFileMode (QFileDialog::ExistingFiles);
	fd->setFilters (filters);
	switch (lmode)
	{
		case NewMap:
			fd->setCaption(vymName+ " - " +tr("Load vym map"));
			break;
		case ImportAdd:
			fd->setCaption(vymName+ " - " +tr("Import: Add vym map to selection"));
			break;
		case ImportReplace:
			fd->setCaption(vymName+ " - " +tr("Import: Replace selection with vym map"));
			break;
	}
	fd->show();

	QString fn;
	if ( fd->exec() == QDialog::Accepted )
	{
		lastFileDir=fd->directory().path();
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		
		progressCounter=flist.count();
		progressCounterTotal=flist.count();
		while( it != flist.end() ) 
		{
			fn = *it;
			fileLoad(*it, lmode);				   
			++it;
		}
	}
	delete (fd);
}

void Main::fileLoad()
{
	fileLoad (NewMap);
}

void Main::fileLoadRecent()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action)
        fileLoad (action->data().toString(), NewMap);
}

void Main::addRecentMap (const QString &fileName)
{

    QStringList files = settings.value("/mainwindow/recentFileList").toStringList();
    files.removeAll(fileName);
    files.prepend(fileName);
    while (files.size() > MaxRecentFiles)
        files.removeLast();

    settings.setValue("/mainwindow/recentFileList", files);

	setupRecentMapsMenu();
}

void Main::fileSave(VymModel *m, const SaveMode &savemode)
{
	if (!m) return;

	if ( m->getFilePath().isEmpty() ) 
	{
		// We have  no filepath yet,
		// call fileSaveAs() now, this will call fileSave() 
		// again.
		// First switch to editor
		//FIXME-3 needed???  tabWidget->setCurrentWidget (m->getMapEditor());
		fileSaveAs(savemode);
	}

	if (m->save (savemode)==success)
	{
		statusBar()->message( 
			tr("Saved  %1").arg(m->getFilePath()), 
			statusbarTime );
		addRecentMap (m->getFilePath() );
	} else		
		statusBar()->message( 
			tr("Couldn't save ").arg(m->getFilePath()), 
			statusbarTime );
}

void Main::fileSave()
{
	fileSave (currentModel(), CompleteMap);
}

void Main::fileSave(VymModel *m)
{
	fileSave (m,CompleteMap);
}

void Main::fileSaveAs(const SaveMode& savemode)
{
	QString fn;

	if (currentMapEditor())
	{
		if (savemode==CompleteMap)
			fn = Q3FileDialog::getSaveFileName( QString::null, "VYM map (*.vym)", this );
		else		
			fn = Q3FileDialog::getSaveFileName( QString::null, "VYM part of map (*.vyp)", this );
		if ( !fn.isEmpty() ) 
		{
			// Check for existing file
			if (QFile (fn).exists())
			{
				QMessageBox mb( vymName,
					tr("The file %1\nexists already. Do you want to").arg(fn),
					QMessageBox::Warning,
					QMessageBox::Yes | QMessageBox::Default,
					QMessageBox::Cancel | QMessageBox::Escape,
					QMessageBox::NoButton);
				mb.setButtonText( QMessageBox::Yes, tr("Overwrite") );
				mb.setButtonText( QMessageBox::Cancel, tr("Cancel"));
				switch( mb.exec() ) 
				{
					case QMessageBox::Yes:
						// save 
						break;
					case QMessageBox::Cancel:
						// do nothing
						return;
						break;
				}
			} else
			{
				// New file, add extension to filename, if missing
				// This is always .vym or .vyp, depending on savemode
				if (savemode==CompleteMap)
				{
					if (!fn.contains (".vym") && !fn.contains (".xml"))
						fn +=".vym";
				} else		
				{
					if (!fn.contains (".vyp") && !fn.contains (".xml"))
						fn +=".vyp";
				}
			}
	



			// Save now
			VymModel *m=currentModel();
			m->setFilePath(fn);
			fileSave(m, savemode);

			// Set name of tab, assuming current tab is the one we just saved
			if (savemode==CompleteMap)
				tabWidget->setTabText (tabWidget->currentIndex(), m->getFileName() );
			return;
		} 
	}
}

void Main::fileSaveAs()
{
	fileSaveAs (CompleteMap);
}

void Main::fileImportKDE3Bookmarks()
{
	ImportKDE3Bookmarks im;
	im.transform();
	if (aborted!=fileLoad (im.getTransformedFile(),NewMap) && currentMapEditor() )
		currentMapEditor()->getModel()->setFilePath ("");
}

void Main::fileImportKDE4Bookmarks()
{
	ImportKDE4Bookmarks im;
	im.transform();
	if (aborted!=fileLoad (im.getTransformedFile(),NewMap) && currentMapEditor() )
		currentMapEditor()->getModel()->setFilePath ("");
}

void Main::fileImportFirefoxBookmarks()
{
	Q3FileDialog *fd=new Q3FileDialog( this);
	fd->setDir (vymBaseDir.homeDirPath()+"/.mozilla/firefox");
	fd->setMode (Q3FileDialog::ExistingFiles);
	fd->addFilter ("Firefox "+tr("Bookmarks")+" (*.html)");
	fd->setCaption(tr("Import")+" "+"Firefox "+tr("Bookmarks"));
	fd->show();

	if ( fd->exec() == QDialog::Accepted )
	{
		ImportFirefoxBookmarks im;
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		while( it != flist.end() ) 
		{
			im.setFile (*it);
			if (im.transform() && 
				aborted!=fileLoad (im.getTransformedFile(),NewMap,FreemindMap) && 
				currentMapEditor() )
				currentMapEditor()->getModel()->setFilePath ("");
			++it;
		}
	}
	delete (fd);
}

void Main::fileImportFreemind()
{
	QStringList filters;
	filters <<"Freemind map (*.mm)"<<"All files (*)";
	QFileDialog *fd=new QFileDialog( this);
	fd->setDir (lastFileDir);
	fd->setFileMode (QFileDialog::ExistingFiles);
	fd->setFilters (filters);
	fd->setCaption(vymName+ " - " +tr("Load Freemind map"));
	fd->show();

	QString fn;
	if ( fd->exec() == QDialog::Accepted )
	{
		lastFileDir=fd->directory().path();
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		while( it != flist.end() ) 
		{
			fn = *it;
			if ( fileLoad (fn,NewMap, FreemindMap)  )
			{	
				currentMapEditor()->getModel()->setFilePath ("");
			}	
			++it;
		}
	}
	delete (fd);
}


void Main::fileImportMM()
{
	ImportMM im;

	Q3FileDialog *fd=new Q3FileDialog( this);
	fd->setDir (lastFileDir);
	fd->setMode (Q3FileDialog::ExistingFiles);
	fd->addFilter ("Mind Manager (*.mmap)");
	fd->setCaption(tr("Import")+" "+"Mind Manager");
	fd->show();

	if ( fd->exec() == QDialog::Accepted )
	{
		lastFileDir=fd->dirPath();
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		while( it != flist.end() ) 
		{
			im.setFile (*it);
			if (im.transform() && 
				success==fileLoad (im.getTransformedFile(),NewMap) && 
				currentMapEditor() )
				currentMapEditor()->getModel()->setFilePath ("");
			++it;
		}
	}
	delete (fd);
}

void Main::fileImportDir()
{
	VymModel *m=currentModel();
	if (m) m->importDir();
}

void Main::fileExportXML()	
{
	VymModel *m=currentModel();
	if (m) m->exportXML();
}

void Main::fileExportHTML()	
{
	VymModel *m=currentModel();
	if (m) m->exportHTML();
}


void Main::fileExportXHTML()	
{
	VymModel *m=currentModel();
	if (m) m->exportXHTML();
}

void Main::fileExportImage()	
{
	VymModel *m=currentModel();
	if (m) m->exportImage();
}

void Main::fileExportAO()
{
	VymModel *m=currentModel();
	if (m) m->exportAO();
}

void Main::fileExportASCII()
{
	VymModel *m=currentModel();
	if (m) m->exportASCII();
}

void Main::fileExportCSV()	//FIXME-3 not scriptable yet
{
	VymModel *m=currentModel();
	if (m)
	{
		ExportCSV ex;
		ex.setModel (m);
		ex.addFilter ("CSV (*.csv)");
		ex.setDir(lastImageDir);
		ex.setCaption(vymName+ " -" +tr("Export as CSV")+" "+tr("(still experimental)"));
		if (ex.execDialog() ) 
		{
			m->setExportMode(true);
			ex.doExport();
			m->setExportMode(false);
		}
	}
}

void Main::fileExportLaTeX()	//FIXME-3 not scriptable yet
{
	VymModel *m=currentModel();
	if (m)
	{
		ExportLaTeX ex;
		ex.setModel (m);
		ex.addFilter ("Tex (*.tex)");
		ex.setDir(lastImageDir);
		ex.setCaption(vymName+ " -" +tr("Export as LaTeX")+" "+tr("(still experimental)"));
		if (ex.execDialog() ) 
		{
			m->setExportMode(true);
			ex.doExport();
			m->setExportMode(false);
		}
	}
}

void Main::fileExportKDE3Bookmarks()	//FIXME-3 not scriptable yet
{
	ExportKDE3Bookmarks ex;
	VymModel *m=currentModel();
	if (m)
	{
		ex.setModel (m);
		ex.doExport();
	}	
}

void Main::fileExportKDE4Bookmarks()	//FIXME-3 not scriptable yet
{
	ExportKDE4Bookmarks ex;
	VymModel *m=currentModel();
	if (m)
	{
		ex.setModel (m);
		ex.doExport();
	}	
}

void Main::fileExportTaskjuggler()	//FIXME-3 not scriptable yet
{
	ExportTaskjuggler ex;
	VymModel *m=currentModel();
	if (m)
	{
		ex.setModel (m);
		ex.setCaption ( vymName+" - "+tr("Export to")+" Taskjuggler"+tr("(still experimental)"));
		ex.setDir(lastImageDir);
		ex.addFilter ("Taskjuggler (*.tjp)");
		if (ex.execDialog() ) 
		{
			m->setExportMode(true);
			ex.doExport();
			m->setExportMode(false);
		}
	}	
}

void Main::fileExportOOPresentation()	//FIXME-3 not scriptable yet
{
	ExportOOFileDialog *fd=new ExportOOFileDialog( this,vymName+" - "+tr("Export to")+" Open Office");
	// TODO add preview in dialog
	//ImagePreview *p =new ImagePreview (fd);
	//fd->setContentsPreviewEnabled( TRUE );
	//fd->setContentsPreview( p, p );
	//fd->setPreviewMode( QFileDialog::Contents );
	fd->setCaption(vymName+" - " +tr("Export to")+" Open Office");
	fd->setDir (QDir().current());
	if (fd->foundConfig())
	{
		fd->show();

		if ( fd->exec() == QDialog::Accepted )
		{
			QString fn=fd->selectedFile();
			if (!fn.contains (".odp"))
				fn +=".odp";

			//lastImageDir=fn.left(fn.findRev ("/"));
			VymModel *m=currentModel();
			if (m) m->exportOOPresentation(fn,fd->selectedConfig());	
		}
	} else
	{
		QMessageBox::warning(0, 
		tr("Warning"),
		tr("Couldn't find configuration for export to Open Office\n"));
	}
}

void Main::fileCloseMap()	
{
	VymModel *m=currentModel();
	if (m)
	{
		if (m->hasChanged())
		{
			QMessageBox mb( vymName,
				tr("The map %1 has been modified but not saved yet. Do you want to").arg(m->getFileName()),
				QMessageBox::Warning,
				QMessageBox::Yes | QMessageBox::Default,
				QMessageBox::No,
				QMessageBox::Cancel | QMessageBox::Escape );
			mb.setButtonText( QMessageBox::Yes, tr("Save modified map before closing it") );
			mb.setButtonText( QMessageBox::No, tr("Discard changes"));
			switch( mb.exec() ) 
			{
				case QMessageBox::Yes:
					// save and close
					fileSave(m, CompleteMap);
					break;
				case QMessageBox::No:
				// close  without saving
					break;
				case QMessageBox::Cancel:
					// do nothing
				return;
			}
		} 
		// And here comes the segfault, because removeTab triggers 
		// FIXME-3 currentChanged->Main::editorChanged -> updateActions and VM is not NULL yet...
		vymViews.removeAt (tabWidget->currentIndex() );
		tabWidget->removeTab (tabWidget->currentIndex() );

		// Remove mapEditor/model FIXME-3   Huh? seems to work now...
		// Better would be delete (me), but then we could have a Qt error:
		// "QObject: Do not delete object, 'MapEditor', during its event handler!"
		// So we only remove data now and call deconstructor when vym closes later
		// this needs to be moved to vymview...   me->clear();
		// some model->clear is needed to free up memory ...

		delete (m->getMapEditor());
		delete (m);

		updateActions();
	}
}

void Main::filePrint()
{
	if (currentMapEditor())
		currentMapEditor()->print();
}

void Main::fileExitVYM()
{
	// Check if one or more editors have changed
	int i;
	for (i=0;i<=vymViews.count() -1;i++)
	{
		// If something changed, ask what to do
		if (vymViews.at(i)->getModel()->hasChanged())
		{
			tabWidget->setCurrentPage(i);
			QMessageBox mb( vymName,
				tr("This map is not saved yet. Do you want to"),
				QMessageBox::Warning,
				QMessageBox::Yes | QMessageBox::Default,
				QMessageBox::No,
				QMessageBox::Cancel | QMessageBox::Escape );
			mb.setButtonText( QMessageBox::Yes, tr("Save map") );
			mb.setButtonText( QMessageBox::No, tr("Discard changes") );
			mb.setModal (true);
			mb.show();
			mb.setActiveWindow();
			switch( mb.exec() ) {
				case QMessageBox::Yes:
					// save (the changed editors) and exit
					fileSave(currentModel(), CompleteMap);
					break;
				case QMessageBox::No:
					// exit without saving
					break;
				case QMessageBox::Cancel:
					// don't save and don't exit
				return;
			}
		}
	} // loop over all MEs	
    qApp->quit();
}

void Main::editUndo()
{
	VymModel *m=currentModel();
	if (m) m->undo();
}

void Main::editRedo()	   
{
	VymModel *m=currentModel();
	if (m) m->redo();
}

void Main::gotoHistoryStep (int i)	   
{
	VymModel *m=currentModel();
	if (m) m->gotoHistoryStep(i);
}

void Main::editCopy()
{
	VymModel *m=currentModel();
	if (m) m->copy();
}

void Main::editPaste()
{
	VymModel *m=currentModel();
	if (m) m->paste();
}

void Main::editCut()
{
	VymModel *m=currentModel();
	if (m) m->cut();
}

void Main::editOpenFindWidget()  
{
	if (!findWidget->isVisible())
	{
		findWidget->show();
		findWidget->setFocus();
	} else if (!findResultWidget->parentWidget()->isVisible())
		findResultWidget->parentWidget()->show();
	else 
	{
		findWidget->hide();
		findResultWidget->parentWidget()->hide();
	}
}

void Main::editHideFindWidget()
{
    // findWidget hides itself, but we want
    // to have focus back at mapEditor usually
	MapEditor *me=currentMapEditor();
    if (me) me->setFocus();
}

void Main::editFindNext(QString s)  
{
	Qt::CaseSensitivity cs=Qt::CaseInsensitive;
	QTextCursor cursor;
	VymModel *m=currentModel();
	if (m) 
	{
		m->findAll (findResultWidget->getResultModel(),s,cs);

		BranchItem *bi=m->findText(s, cs,cursor);
		if (bi)
		{
			findWidget->setStatus (FindWidget::Success);
		}	
		else
			findWidget->setStatus (FindWidget::Failed);
	}
}

void Main::editFindDuplicateURLs() //FIXME-4 feature: use FindResultWidget for display
{
	VymModel *m=currentModel();
	if (m) m->findDuplicateURLs();
}

void Main::openTabs(QStringList urls)
{
	if (!urls.isEmpty())
	{	
		bool success=true;
		QStringList args;
		QString browser=settings.value("/mainwindow/readerURL" ).toString();
		//qDebug ()<<"Services: "<<QDBusConnection::sessionBus().interface()->registeredServiceNames().value();
		if (*browserPID==0 ||
			(browser.contains("konqueror") &&
			 !QDBusConnection::sessionBus().interface()->registeredServiceNames().value().contains (QString("org.kde.konqueror-%1").arg(*browserPID)))
		   )	 
		{
			// Start a new browser, if there is not one running already or
			// if a previously started konqueror is gone.
			if (debug) cout <<"Main::openTabs no konqueror-"<<*browserPID<<" found\n";
			QString u=urls.takeFirst();
			args<<u;
			QString workDir=QDir::currentDirPath();
			if (!QProcess::startDetached(browser,args,workDir,browserPID))
			{
				// try to set path to browser
				QMessageBox::warning(0, 
					tr("Warning"),
					tr("Couldn't find a viewer to open %1.\n").arg(u)+
					tr("Please use Settings->")+tr("Set application to open an URL"));
				return;
			}
			if (debug) cout << "Main::openTabs  Started konqueror-"<<*browserPID<<endl;
#if defined(Q_OS_WIN32)
            // There's no sleep in VCEE, replace it with Qt's QThread::wait().
            this->thread()->wait(3000);
#else
			sleep (3);	//FIXME-3 needed?
#endif
		}

		if (browser.contains("konqueror"))
		{
			for (int i=0; i<urls.size(); i++)
			{
				// Open new browser
				// Try to open new tab in existing konqueror started previously by vym
				args.clear();

/*	On KDE3 use DCOP
				args<< QString("konqueror-%1").arg(procBrowser->pid())<<
					"konqueror-mainwindow#1"<<
					"newTab" <<
					urls.at(i);
*/					
				args<< QString("org.kde.konqueror-%1").arg(*browserPID)<<
					"/konqueror/MainWindow_1"<<
					"newTab" <<
					urls.at(i)<<
					"false";
				if (debug) cout << "MainWindow::openURLs  args="<<args.join(" ").toStdString()<<endl;
				if (!QProcess::startDetached ("qdbus",args))
					success=false;
			}
			if (!success)
				QMessageBox::warning(0, 
					tr("Warning"),
					tr("Couldn't start %1 to open a new tab in %2.").arg("dcop").arg("konqueror"));
			return;		
		} else if (browser.contains ("firefox") || browser.contains ("mozilla") )
		{
			for (int i=0; i<urls.size(); i++)
			{
				// Try to open new tab in firefox
				args<< "-remote"<< QString("openurl(%1,new-tab)").arg(urls.at(i));
				if (!QProcess::startDetached (browser,args))
					success=false;
			}			
			if (!success)
				QMessageBox::warning(0, 
					tr("Warning"),
					tr("Couldn't start %1 to open a new tab").arg(browser));
			return;		
		}			
		QMessageBox::warning(0, 
			tr("Warning"),
			tr("Sorry, currently only Konqueror and Mozilla support tabbed browsing."));
	}	
}

void Main::editOpenURL()
{
	// Open new browser
	VymModel *m=currentModel();
	if (m)
	{	
	    QString url=m->getURL();
		QStringList args;
		if (url=="") return;
		QString browser=settings.value("/mainwindow/readerURL" ).toString();
		args<<url;
		QString workDir=QDir::currentDirPath();
		if (!procBrowser->startDetached(browser,args))
		{
			// try to set path to browser
			QMessageBox::warning(0, 
				tr("Warning"),
				tr("Couldn't find a viewer to open %1.\n").arg(url)+
				tr("Please use Settings->")+tr("Set application to open an URL"));
			settingsURL() ; 
		}	
	}	
}
void Main::editOpenURLTab()
{
	VymModel *m=currentModel();
	if (m)
	{	
	    QStringList urls;
		urls.append(m->getURL());
		openTabs (urls);
	}	
}

void Main::editOpenMultipleVisURLTabs(bool ignoreScrolled)
{
	VymModel *m=currentModel();
	if (m)
	{	
	    QStringList urls;
		urls=m->getURLs(ignoreScrolled);
		openTabs (urls);
	}	
}

void Main::editOpenMultipleURLTabs()
{
	editOpenMultipleVisURLTabs (false);
}


void Main::editURL()
{
	VymModel *m=currentModel();
	if (m) m->editURL();
}

void Main::editLocalURL()
{
	VymModel *m=currentModel();
	if (m) m->editLocalURL();
}

void Main::editHeading2URL()
{
	VymModel *m=currentModel();
	if (m) m->editHeading2URL();
}

void Main::editBugzilla2URL()
{
	VymModel *m=currentModel();
	if (m) m->editBugzilla2URL();
}

void Main::getBugzillaData()
{
	VymModel *m=currentModel();
	/*
	QProgressDialog progress ("Doing stuff","cancl",0,10,this);
	progress.setWindowModality(Qt::WindowModal);
	//progress.setCancelButton (NULL);
	progress.show();
	progress.setMinimumDuration (0);
	progress.setValue (1);
	progress.setValue (5);
	progress.update();
	*/
	/*
	QProgressBar *pb=new QProgressBar;
	pb->setMinimum (0);
	pb->setMaximum (0);
	pb->show();
	pb->repaint();
	*/
	if (m) m->getBugzillaData();
}

void Main::editFATE2URL()
{
	VymModel *m=currentModel();
	if (m) m->editFATE2URL();
}

void Main::editHeadingFinished(VymModel *m)
{
	if (m)
	{
		if (!actionSettingsAutoSelectNewBranch->isOn() && 
			!prevSelection.isEmpty()) 
			m->select(prevSelection);
		prevSelection="";
	}
}

void Main::openVymLinks(const QStringList &vl)
{
	for (int j=0; j<vl.size(); j++)
	{
		// compare path with already loaded maps
		int index=-1;
		int i;
		for (i=0;i<=vymViews.count() -1;i++)
		{
			if (vl.at(j)==vymViews.at(i)->getModel()->getFilePath() )
			{
				index=i;
				break;
			}
		}	
		if (index<0)
		// Load map
		{
			if (!QFile(vl.at(j)).exists() )
				QMessageBox::critical( 0, tr( "Critical Error" ),
				   tr("Couldn't open map %1").arg(vl.at(j)));
			else
			{
				fileLoad (vl.at(j), NewMap);
				tabWidget->setCurrentIndex (tabWidget->count()-1);	
			}
		} else
			// Go to tab containing the map
			tabWidget->setCurrentIndex (index);	
	}
}

void Main::editOpenVymLink()
{
	VymModel *m=currentModel();
	if (m)
	{
		QStringList vl;
		vl.append(m->getVymLink());	
		openVymLinks (vl);
	}
}

void Main::editOpenMultipleVymLinks()
{
	QString currentVymLink;
	VymModel *m=currentModel();
	if (m)
	{
		QStringList vl=m->getVymLinks();
		openVymLinks (vl);
	}
}

void Main::editVymLink()
{
	VymModel *m=currentModel();
	if (m)
		m->editVymLink();	
}

void Main::editDeleteVymLink()
{
	VymModel *m=currentModel();
	if (m) m->deleteVymLink();	
}

void Main::editToggleHideExport()
{
	VymModel *m=currentModel();
	if (m) m->toggleHideExport();	
}

void Main::editAddTimestamp()
{
	VymModel *m=currentModel();
	if (m) m->addTimestamp();	
}

void Main::editMapInfo()
{
	VymModel *m=currentModel();
	if (!m) return;

	ExtraInfoDialog dia;
	dia.setMapName (m->getFileName() );
	dia.setAuthor (m->getAuthor() );
	dia.setComment(m->getComment() );

	// Calc some stats
	QString stats;
    stats+=tr("%1 items on map\n","Info about map").arg (m->getScene()->items().size(),6);

	uint b=0;
	uint f=0;
	uint n=0;
	uint xl=0;
	BranchItem *cur=NULL;
	BranchItem *prev=NULL;
	m->nextBranch(cur,prev);
	while (cur) 
	{
		if (!cur->getNote().isEmpty() ) n++;
		f+= cur->imageCount();
		b++;
		xl+=cur->xlinkCount();
		m->nextBranch(cur,prev);
	}

    stats+=QString ("%1 xLinks \n").arg (xl,6);
    stats+=QString ("%1 notes\n").arg (n,6);
    stats+=QString ("%1 images\n").arg (f,6);
    stats+=QString ("%1 branches\n").arg (m->branchCount(),6);
	dia.setStats (stats);

	// Finally show dialog
	if (dia.exec() == QDialog::Accepted)
	{
		m->setAuthor (dia.getAuthor() );
		m->setComment (dia.getComment() );
	}
}

void Main::editMoveUp()
{
	VymModel *m=currentModel();
	if (m) m->moveUp();
}

void Main::editMoveDown()
{
	VymModel *m=currentModel();
	if (m) m->moveDown();
}

void Main::editDetach()
{
	VymModel *m=currentModel();
	if (m) m->detach();
}

void Main::editSortChildren()
{
	VymModel *m=currentModel();
	if (m) m->sortChildren(false);
}

void Main::editSortBackChildren()
{
	VymModel *m=currentModel();
	if (m) m->sortChildren(true);
}

void Main::editToggleScroll()
{
	VymModel *m=currentModel();
	if (m) m->toggleScroll();
}

void Main::editExpandAll()
{
	VymModel *m=currentModel();
	if (m) m->emitExpandAll();
}

void Main::editExpandOneLevel()
{
	VymModel *m=currentModel();
	if (m) m->emitExpandOneLevel();
}

void Main::editCollapseOneLevel()
{
	VymModel *m=currentModel();
	if (m) m->emitCollapseOneLevel();
}

void Main::editUnscrollChildren()
{
	VymModel *m=currentModel();
	if (m) m->unscrollChildren();
}

void Main::editAddAttribute()
{
	VymModel *m=currentModel();
	if (m) 
	{

		m->addAttribute();
	}
}

void Main::editAddMapCenter()
{
	VymModel *m=currentModel();
	if (m) m->select (m->addMapCenter ());
}

void Main::editNewBranch()
{
	VymModel *m=currentModel();
	if (m)
	{
		BranchItem *bi=m->addNewBranch();
		if (!bi) return;

		if (actionSettingsAutoEditNewBranch->isOn() 
		     && !actionSettingsAutoSelectNewBranch->isOn() )
			prevSelection=m->getSelectString();
		else	
			prevSelection=QString();

		if (actionSettingsAutoSelectNewBranch->isOn()  
			|| actionSettingsAutoEditNewBranch->isOn()) 
		{
			m->select (bi);
			if (actionSettingsAutoEditNewBranch->isOn())
				currentMapEditor()->editHeading();
		}
	}	
}

void Main::editNewBranchBefore()
{
	VymModel *m=currentModel();
	if (m)
	{
		BranchItem *bi=m->addNewBranchBefore();

		if (bi) 
			m->select (bi);
		else
			return;

		if (actionSettingsAutoEditNewBranch->isOn())
		{
			if (!actionSettingsAutoSelectNewBranch->isOn())
				prevSelection=m->getSelectString(bi); 
			currentMapEditor()->editHeading();
		}
	}	
}

void Main::editNewBranchAbove()	
{
	VymModel *m=currentModel();
	if ( m)
	{
		BranchItem *bi=m->addNewBranch (-1);


		if (bi) 
			m->select (bi);
		else
			return;

		if (actionSettingsAutoEditNewBranch->isOn())
		{
			if (!actionSettingsAutoSelectNewBranch->isOn())
				prevSelection=m->getSelectString (bi);	
			currentMapEditor()->editHeading();
		}
	}	
}

void Main::editNewBranchBelow()
{
	VymModel *m=currentModel();
	if (m)
	{
		BranchItem *bi=m->addNewBranch (1);

		if (bi) 
			m->select (bi);
		else
			return;

		if (actionSettingsAutoEditNewBranch->isOn())
		{
			if (!actionSettingsAutoSelectNewBranch->isOn())
				prevSelection=m->getSelectString(bi);
			currentMapEditor()->editHeading();
		}
	}	
}

void Main::editImportAdd()
{
	fileLoad (ImportAdd);
}

void Main::editImportReplace()
{
	fileLoad (ImportReplace);
}

void Main::editSaveBranch()
{
	fileSaveAs (PartOfMap);
}

void Main::editDeleteKeepChildren()
{
	VymModel *m=currentModel();
	 if (m) m->deleteKeepChildren();
}

void Main::editDeleteChildren()
{
	VymModel *m=currentModel();
	if (m) m->deleteChildren();
}

void Main::editDeleteSelection()
{
	VymModel *m=currentModel();
	if (m && actionSettingsUseDelKey->isOn())
		m->deleteSelection();
}

void Main::editLoadImage()
{
	VymModel *m=currentModel();
	if (m) m->loadFloatImage();
}

void Main::editSaveImage()
{
	VymModel *m=currentModel();
	if (m) m->saveFloatImage();
}

void Main::editEditXLink(QAction *a)
{
	VymModel *m=currentModel();
	if (m)
		m->editXLink(branchXLinksContextMenuEdit->actions().indexOf(a));
}

void Main::formatSelectColor()
{
	QColor col = QColorDialog::getColor((currentColor ), this );
	if ( !col.isValid() ) return;
	colorChanged( col );
}

void Main::formatPickColor()
{
	VymModel *m=currentModel();
	if (m)
		colorChanged( m->getCurrentHeadingColor() );
}

void Main::colorChanged(QColor c)
{
    QPixmap pix( 16, 16 );
    pix.fill( c );
    actionFormatColor->setIconSet( pix );
	currentColor=c;
}

void Main::formatColorBranch()
{
	VymModel *m=currentModel();
	if (m) m->colorBranch(currentColor);
}

void Main::formatColorSubtree()
{
	VymModel *m=currentModel();
	if (m) m->colorSubtree (currentColor);
}

void Main::formatLinkStyleLine()
{
	VymModel *m=currentModel();
	if (m)
    {
		m->setMapLinkStyle("StyleLine");
        actionFormatLinkStyleLine->setChecked(true);
    }
}

void Main::formatLinkStyleParabel()
{
	VymModel *m=currentModel();
	if (m)
    {
		m->setMapLinkStyle("StyleParabel");
        actionFormatLinkStyleParabel->setChecked(true);
    }
}

void Main::formatLinkStylePolyLine()
{
	VymModel *m=currentModel();
	if (m)
    {
		m->setMapLinkStyle("StylePolyLine");
        actionFormatLinkStylePolyLine->setChecked(true);
    }
}

void Main::formatLinkStylePolyParabel()
{
	VymModel *m=currentModel();
	if (m)
    {
		m->setMapLinkStyle("StylePolyParabel");
        actionFormatLinkStylePolyParabel->setChecked(true);
    }
}

void Main::formatSelectBackColor()
{
	VymModel *m=currentModel();
	if (m) m->selectMapBackgroundColor();
}

void Main::formatSelectBackImage()
{
	VymModel *m=currentModel();
	if (m)
		m->selectMapBackgroundImage();
}

void Main::formatSelectLinkColor()
{
	VymModel *m=currentModel();
	if (m)
	{
		QColor col = QColorDialog::getColor( m->getMapDefLinkColor(), this );
		m->setMapDefLinkColor( col );
	}
}

void Main::formatSelectSelectionColor()
{
	VymModel *m=currentModel();
	if (m)
	{
		QColor col = QColorDialog::getColor( m->getMapDefLinkColor(), this );
		m->setSelectionColor (col);
	}

}

void Main::formatToggleLinkColorHint()
{
	VymModel *m=currentModel();
	if (m) m->toggleMapLinkColorHint();
}


void Main::formatHideLinkUnselected()	//FIXME-3 get rid of this with imagepropertydialog
{
	VymModel *m=currentModel();
	if (m)
		m->setHideLinkUnselected(actionFormatHideLinkUnselected->isOn());
}

void Main::viewZoomReset()
{
	MapEditor *me=currentMapEditor();
	if (me) me->setZoomFactorTarget (1);
}

void Main::viewZoomIn()
{
	MapEditor *me=currentMapEditor();
	if (me) me->setZoomFactorTarget (me->getZoomFactorTarget()*1.15);
}

void Main::viewZoomOut()
{
	MapEditor *me=currentMapEditor();
	if (me) me->setZoomFactorTarget (me->getZoomFactorTarget()*0.85);
}

void Main::viewCenter()
{
	VymModel *m=currentModel();
	if (m) m->emitShowSelection();
}

void Main::networkStartServer()
{
	VymModel *m=currentModel();
	if (m) m->newServer();
}

void Main::networkConnect()
{
	VymModel *m=currentModel();
	if (m) m->connectToServer();
}

bool Main::settingsPDF()
{
	// Default browser is set in constructor
	bool ok;
	QString text = QInputDialog::getText(
		"VYM", tr("Set application to open PDF files")+":", QLineEdit::Normal,
		settings.value("/mainwindow/readerPDF").toString(), &ok, this );
	if (ok)
		settings.setValue ("/mainwindow/readerPDF",text);
	return ok;
}


bool Main::settingsURL()
{
	// Default browser is set in constructor
	bool ok;
	QString text = QInputDialog::getText(
		"VYM", tr("Set application to open an URL")+":", QLineEdit::Normal,
		settings.value("/mainwindow/readerURL").toString()
		, &ok, this );
	if (ok)
		settings.setValue ("/mainwindow/readerURL",text);
	return ok;
}

void Main::settingsMacroDir()
{
	QDir defdir(vymBaseDir.path() + "/macros");
	if (!defdir.exists())
		defdir=vymBaseDir;
	QDir dir=QFileDialog::getExistingDirectory (
		this,
		tr ("Directory with vym macros:"), 
		settings.value ("/macros/macroDir",defdir.path()).toString()
	);
	if (dir.exists())
		settings.setValue ("/macros/macroDir",dir.absolutePath());
}

void Main::settingsUndoLevels()
{
	bool ok;
	int i = QInputDialog::getInteger(
		this, 
		tr("QInputDialog::getInteger()"),
	    tr("Number of undo/redo levels:"), settings.value("/mapeditor/stepsTotal").toInt(), 0, 1000, 1, &ok);
	if (ok)
	{
		settings.setValue ("/mapeditor/stepsTotal",i);
		QMessageBox::information( this, tr( "VYM -Information:" ),
		   tr("Settings have been changed. The next map opened will have \"%1\" undo/redo levels").arg(i)); 
   }	
}

void Main::settingsAutosaveToggle()
{
	settings.setValue ("/mainwindow/autosave/use",actionSettingsAutosaveToggle->isOn() );
}

void Main::settingsAutosaveTime()
{
	bool ok;
	int i = QInputDialog::getInteger(
		this, 
		tr("QInputDialog::getInteger()"),
	    tr("Number of seconds before autosave:"), settings.value("/mainwindow/autosave/ms").toInt() / 1000, 10, 10000, 1, &ok);
	if (ok)
		settings.setValue ("/mainwindow/autosave/ms",i * 1000);
}

void Main::settingsAutoLayoutToggle()
{
	settings.setValue ("/mainwindow/autoLayout/use",actionSettingsAutosaveToggle->isOn() );
}

void Main::settingsWriteBackupFileToggle()
{
	settings.setValue ("/mainwindow/writeBackupFile",actionSettingsWriteBackupFile->isOn() );
}

void Main::settingsToggleAnimation()
{
	settings.setValue ("/animation/use",actionSettingsUseAnimation->isOn() );
}

void Main::settingsToggleDelKey()
{
	if (actionSettingsUseDelKey->isOn())
	{
		actionDelete->setAccel (QKeySequence (Qt::Key_Delete));
	} else
	{
		actionDelete->setAccel (QKeySequence (""));
	}
}

void Main::windowToggleNoteEditor()
{
	if (textEditor->isVisible() )
		windowHideNoteEditor();
	else
		windowShowNoteEditor();
}

void Main::windowToggleTreeEditor()
{
	if ( tabWidget->currentPage())
		vymViews.at(tabWidget->currentIndex())->toggleTreeEditor();
}

void Main::windowToggleHistory()
{
	if (historyWindow->isVisible())
		historyWindow->hide();
	else	
		historyWindow->show();

}

void Main::windowToggleProperty()
{
	if (branchPropertyWindow->isVisible())
		branchPropertyWindow->hide();
	else	
		branchPropertyWindow->show();
	branchPropertyWindow->setModel (currentModel() );
}

void Main::windowToggleAntiAlias()
{
	bool b=actionViewToggleAntiAlias->isOn();
	MapEditor *me;
	for (int i=0;i<vymViews.count();i++)
	{
		me=vymViews.at(i)->getMapEditor();
		if (me) me->setAntiAlias(b);
	}	

}

bool Main::isAliased()
{
	return actionViewToggleAntiAlias->isOn();
}

bool Main::hasSmoothPixmapTransform()
{
	return actionViewToggleSmoothPixmapTransform->isOn();
}

void Main::windowToggleSmoothPixmap()
{
	bool b=actionViewToggleSmoothPixmapTransform->isOn();
	MapEditor *me;
	for (int i=0;i<vymViews.count();i++)
	{
		
		me=vymViews.at(i)->getMapEditor();
		if (me) me->setSmoothPixmap(b);
	}	
}

void Main::updateHistory(SimpleSettings &undoSet)
{
	historyWindow->update (undoSet);
}

void Main::updateNoteFlag()	
{
	// this slot is connected to TextEditor::textHasChanged()
	VymModel *m=currentModel();
	if (m) m->updateNoteFlag();
}

void Main::updateNoteEditor(QModelIndex index )
{
	TreeItem *ti=((VymModel*) QObject::sender())->getItem(index);
	/*
	cout << "Main::updateNoteEditor model="<<sender();
	cout << "  item="<<ti->getHeadingStd()<<" ("<<ti<<")"<<endl;
	*/
	textEditor->setNote (ti->getNoteObj() );
}

void Main::changeSelection (VymModel *model, const QItemSelection &newsel, const QItemSelection &oldsel)
{
	branchPropertyWindow->setModel (model ); //FIXME-3 this used to be called from BranchObj::select(). Maybe use signal now...

	if (model && model==currentModel() )
	{
		// NoteEditor
		TreeItem *ti;
		if (!oldsel.indexes().isEmpty() )
		{
			ti=model->getItem(oldsel.indexes().first());

			// Don't update note if both treeItem and textEditor are empty
			//if (! (ti->hasEmptyNote() && textEditor->isEmpty() ))
			//	ti->setNoteObj (textEditor->getNoteObj(),false );
		} 
		if (!newsel.indexes().isEmpty() )
		{
			ti=model->getItem(newsel.indexes().first());
			if (!ti->hasEmptyNote() )
				textEditor->setNote(ti->getNoteObj() );
			else
				textEditor->setNote(NoteObj() );	//FIXME-4 maybe add a clear() to TE
			
			// Show URL and link in statusbar	
			QString status;
			QString s=ti->getURL();
			if (!s.isEmpty() ) status+="URL: "+s+"  ";
			s=ti->getVymLink();
			if (!s.isEmpty() ) status+="Link: "+s;
			if (!status.isEmpty() ) statusMessage (status);

		} else
			textEditor->setInactive();

		updateActions();
	}
}

void Main::updateActions()
{
	// updateActions is also called when satellites are closed	//FIXME-2 doesn't update immediatly, e.g. historyWindow is still visible, when "close" is pressed
	actionViewToggleNoteEditor->setChecked (textEditor->isVisible());
	actionViewToggleHistoryWindow->setChecked (historyWindow->isVisible());
	actionViewTogglePropertyWindow->setChecked (branchPropertyWindow->isVisible());

	VymModel  *m =currentModel();
	if (m) 
	{
		// Printing
		actionFilePrint->setEnabled (true);

		// Link style in context menu
		switch (m->getMapLinkStyle())
		{
			case LinkableMapObj::Line: 
				actionFormatLinkStyleLine->setChecked(true);
				break;
			case LinkableMapObj::Parabel:
				actionFormatLinkStyleParabel->setChecked(true);
				break;
			case LinkableMapObj::PolyLine:	
				actionFormatLinkStylePolyLine->setChecked(true);
				break;
			case LinkableMapObj::PolyParabel:	
				actionFormatLinkStylePolyParabel->setChecked(true);
				break;
			default:
				break;
		}		

		// Update colors
		QPixmap pix( 16, 16 );
		pix.fill( m->getMapBackgroundColor() );
		actionFormatBackColor->setIconSet( pix );
		pix.fill( m->getSelectionColor() );
		actionFormatSelectionColor->setIconSet( pix );
		pix.fill( m->getMapDefLinkColor() );
		actionFormatLinkColor->setIconSet( pix );

		// History window
		historyWindow->setCaption (vymName + " - " +tr("History for %1","Window Caption").arg(m->getFileName()));


		// Expanding/collapsing
		actionExpandAll->setEnabled (true);
		actionExpandOneLevel->setEnabled (true);
		actionCollapseOneLevel->setEnabled (true);
	} else
	{
		// Printing
		actionFilePrint->setEnabled (false);

		// Expanding/collapsing
		actionExpandAll->setEnabled (false);
		actionExpandOneLevel->setEnabled (false);
		actionCollapseOneLevel->setEnabled (false);
	}

	if (m && m->getMapLinkColorHint()==LinkableMapObj::HeadingColor) 
		actionFormatLinkColorHint->setChecked(true);
	else	
		actionFormatLinkColorHint->setChecked(false);


	if (m && m->hasChanged() )
		actionFileSave->setEnabled( true);
	else	
		actionFileSave->setEnabled( false);
	if (m && m->isUndoAvailable())
		actionUndo->setEnabled( true);
	else	
		actionUndo->setEnabled( false);

	if (m && m->isRedoAvailable())
		actionRedo->setEnabled( true);
	else	
		actionRedo->setEnabled( false);

	if (m)
	{
		TreeItem *selti=m->getSelectedItem();
		BranchItem *selbi=m->getSelectedBranch();
		if (selti)
		{
			if (selbi || selti->getType()==TreeItem::Image)
			{
				actionFormatHideLinkUnselected->setChecked (((MapItem*)selti)->getHideLinkUnselected());
				actionFormatHideLinkUnselected->setEnabled (true);
			}

			if (selbi)	
			{
				// Take care of xlinks  
				branchXLinksContextMenuEdit->clear();
				if (selbi->xlinkCount()>0)
				{
					BranchItem *bi;
					QString s;
					for (int i=0; i<selbi->xlinkCount();++i)
					{
						bi=selbi->getXLinkNum(i)->getPartnerBranch();
						if (bi)
						{
							s=bi->getHeading();
							if (s.length()>xLinkMenuWidth)
								s=s.left(xLinkMenuWidth)+"...";
							branchXLinksContextMenuEdit->addAction (s);
						}	
					}
				}
				//Standard Flags
				standardFlagsMaster->updateToolBar (selbi->activeStandardFlagNames() );

				// System Flags
				actionToggleScroll->setEnabled (true);
				if ( selbi->isScrolled() )
					actionToggleScroll->setChecked(true);
				else	
					actionToggleScroll->setChecked(false);

				if ( selti->getURL().isEmpty() )
				{
					actionOpenURL->setEnabled (false);
					actionOpenURLTab->setEnabled (false);
				}	
				else	
				{
					actionOpenURL->setEnabled (true);
					actionOpenURLTab->setEnabled (true);
				}
				if ( selti->getVymLink().isEmpty() )
				{
					actionOpenVymLink->setEnabled (false);
					actionDeleteVymLink->setEnabled (false);
				} else	
				{
					actionOpenVymLink->setEnabled (true);
					actionDeleteVymLink->setEnabled (true);
				}	

				if (selbi->canMoveUp()) 
					actionMoveUp->setEnabled (true);
				else	
					actionMoveUp->setEnabled (false);
				if (selbi->canMoveDown()) 
					actionMoveDown->setEnabled (true);
				else	
					actionMoveDown->setEnabled (false);

				actionSortChildren->setEnabled (true);
				actionSortBackChildren->setEnabled (true);

				actionToggleHideExport->setEnabled (true);	
				actionToggleHideExport->setChecked (selbi->hideInExport() );	

				actionCopy->setEnabled (true);	
				actionCut->setEnabled (true);	
				if (!clipboardEmpty)
					actionPaste->setEnabled (true);	
				else	
					actionPaste->setEnabled (false);	
				for (int i=0; i<actionListBranches.size(); ++i)	
					actionListBranches.at(i)->setEnabled(true);
				actionDelete->setEnabled (true);
			}	// Branch
			if ( selti->getType()==TreeItem::Image)
			{
				actionOpenURL->setEnabled (false);
				actionOpenVymLink->setEnabled (false);
				actionDeleteVymLink->setEnabled (false);	
				actionToggleHideExport->setEnabled (true);	
				actionToggleHideExport->setChecked (selti->hideInExport() );	


				actionCopy->setEnabled (true);
				actionCut->setEnabled (true);	
				actionPaste->setEnabled (false);	//FIXME-4 why not allowing copy of images?
				for (int i=0; i<actionListBranches.size(); ++i)	
					actionListBranches.at(i)->setEnabled(false);
				actionDelete->setEnabled (true);
				actionMoveUp->setEnabled (false);
				actionMoveDown->setEnabled (false);
			}	// Image

		} else
		{	// !selti
			actionCopy->setEnabled (false);	
			actionCut->setEnabled (false);	
			actionPaste->setEnabled (false);	
			for (int i=0; i<actionListBranches.size(); ++i)	
				actionListBranches.at(i)->setEnabled(false);

			actionToggleScroll->setEnabled (false);
			actionOpenURL->setEnabled (false);
			actionOpenVymLink->setEnabled (false);
			actionDeleteVymLink->setEnabled (false);	
			actionHeading2URL->setEnabled (false);	
			actionDelete->setEnabled (false);
			actionMoveUp->setEnabled (false);
			actionMoveDown->setEnabled (false);
			actionFormatHideLinkUnselected->setEnabled (false);
			actionSortChildren->setEnabled (false);
			actionSortBackChildren->setEnabled (false);
			actionToggleHideExport->setEnabled (false);	
		}	
	} // m
}

Main::ModMode Main::getModMode()
{
	if (actionModModeColor->isOn()) return ModModeColor;
	if (actionModModeCopy->isOn()) return ModModeCopy;
	if (actionModModeXLink->isOn()) return ModModeXLink;
	return ModModeNone;
}

bool Main::autoEditNewBranch()
{
	return actionSettingsAutoEditNewBranch->isOn();
}

bool Main::autoSelectNewBranch()
{
	return actionSettingsAutoSelectNewBranch->isOn();
}

void Main::windowShowNoteEditor()
{
	textEditor->setShowWithMain(true);
	textEditor->show();
	actionViewToggleNoteEditor->setChecked (true);
}

void Main::windowHideNoteEditor()
{
	textEditor->setShowWithMain(false);
	textEditor->hide();
	actionViewToggleNoteEditor->setChecked (false);
}

void Main::setScript (const QString &script)
{
	scriptEditor->setScript (script);
}

void Main::runScript (const QString &script)
{
	VymModel *m=currentModel();
	if (m) m->runScript (script);
}

void Main::runScriptEverywhere (const QString &script)
{
	MapEditor *me;
	for (int i=0;i<=tabWidget->count() -1;i++)
	{
		me=(MapEditor*)tabWidget->page(i);
		if (me) me->getModel()->runScript (script);
	}	
}

void Main::windowNextEditor()
{
	if (tabWidget->currentIndex() < tabWidget->count())
		tabWidget->setCurrentIndex (tabWidget->currentIndex() +1);
}

void Main::windowPreviousEditor()
{
	if (tabWidget->currentIndex() >0)
		tabWidget->setCurrentIndex (tabWidget->currentIndex() -1);
}

void Main::standardFlagChanged()
{
	if (currentModel())
	{
		if ( actionSettingsUseFlagGroups->isOn() )
			currentModel()->toggleStandardFlag(sender()->name(),standardFlagsMaster);
		else	
			currentModel()->toggleStandardFlag(sender()->name());
		updateActions();	
	}
}


#include "attributeitem.h"
void Main::testFunction1()
{
	VymModel *m=currentModel();
	if (!m) return;

	BranchItem *selbi=m->getSelectedBranch();
	if (selbi)
	{
		QList<QVariant> cData;
		cData << "new ai" << "undef";

		AttributeItem *ai=new AttributeItem (cData,selbi);
		ai->set ("Key 1","Val a",AttributeItem::FreeString);

		m->addAttribute (ai);
	}
	return;

/*
	if (!currentMapEditor()) return;
	currentMapEditor()->testFunction1();
*/
}

void Main::testFunction2()
{
	if (!currentMapEditor()) return;
	currentMapEditor()->testFunction2();
}

void Main::testCommand()
{
	if (!currentMapEditor()) return;
	scriptEditor->show();
	/*
	bool ok;
	QString com = QInputDialog::getText(
			vymName, "Enter Command:", QLineEdit::Normal,"command", &ok, this );
	if (ok) currentMapEditor()->parseAtom(com);
	*/
}

void Main::helpDoc()
{
	QString locale = QLocale::system().name();
	QString docname;
	if (locale.left(2)=="es")
		docname="vym_es.pdf";
	else	
		docname="vym.pdf";

	QStringList searchList;
	QDir docdir;
	#if defined(Q_OS_MACX)
		searchList << "./vym.app/Contents/Resources/doc";
    #elif defined(Q_OS_WIN32)
        searchList << vymInstallDir.path() + "/share/doc/packages/vym";
	#else
		#if defined(VYM_DOCDIR)
			searchList << VYM_DOCDIR;
		#endif
		// default path in SUSE LINUX
		searchList << "/usr/share/doc/packages/vym";
	#endif

	searchList << "doc";	// relative path for easy testing in tarball
	searchList << "doc/tex";	// Easy testing working on vym.tex
	searchList << "/usr/share/doc/vym";	// Debian
	searchList << "/usr/share/doc/packages";// Knoppix

	bool found=false;
	QFile docfile;
	for (int i=0; i<searchList.count(); ++i)
	{
		docfile.setFileName(searchList.at(i)+"/"+docname);
		if (docfile.exists())
		{
			found=true;
			break;
		}	
	}

	if (!found)
	{
		QMessageBox::critical(0, 
			tr("Critcal error"),
			tr("Couldn't find the documentation %1 in:\n%2").arg(searchList.join("\n")));
		return;
	}	

	QStringList args;
	Process *pdfProc = new Process();
    args << QDir::toNativeSeparators(docfile.fileName());

	if (!pdfProc->startDetached( settings.value("/mainwindow/readerPDF").toString(),args) )
	{
		// error handling
		QMessageBox::warning(0, 
			tr("Warning"),
			tr("Couldn't find a viewer to open %1.\n").arg(docfile.fileName())+
			tr("Please use Settings->")+tr("Set application to open PDF files"));
		settingsPDF();	
		return;
	}
}


void Main::helpDemo()
{
	QStringList filters;
	filters <<"VYM example map (*.vym)";
	QFileDialog *fd=new QFileDialog( this);
	#if defined(Q_OS_MACX)
		fd->setDir (QDir("./vym.app/Contents/Resources/demos"));
	#else
		// default path in SUSE LINUX
		fd->setDir (QDir(vymBaseDir.path()+"/demos"));
	#endif

	fd->setFileMode (QFileDialog::ExistingFiles);
	fd->setFilters (filters);
	fd->setCaption(vymName+ " - " +tr("Load vym example map"));
	fd->show();

	QString fn;
	if ( fd->exec() == QDialog::Accepted )
	{
		lastFileDir=fd->directory().path();
	    QStringList flist = fd->selectedFiles();
		QStringList::Iterator it = flist.begin();
		while( it != flist.end() ) 
		{
			fn = *it;
			fileLoad(*it, NewMap);				   
			++it;
		}
	}
	delete (fd);
}


void Main::helpAbout()
{
	AboutDialog ad;
	ad.setName ("aboutwindow");
	ad.setMinimumSize(500,500);
	ad.resize (QSize (500,500));
	ad.exec();
}

void Main::helpAboutQT()
{
	QMessageBox::aboutQt( this, "Qt Application Example" );
}

void Main::callMacro ()
{
    QAction *action = qobject_cast<QAction *>(sender());
	int i=-1;
    if (action)
	{
        i=action->data().toInt();
		QString mDir (settings.value ("macros/macroDir").toString() );

		QString fn=mDir + QString("/macro-%1.vys").arg(i+1);
		QFile f (fn);
		if ( !f.open( QIODevice::ReadOnly ) )
		{
			QMessageBox::warning(0, 
				tr("Warning"),
				tr("Couldn't find a macro at  %1.\n").arg(fn)+
				tr("Please use Settings->")+tr("Set directory for vym macros"));
			return;
		}	

		QTextStream ts( &f );
		QString macro= ts.read();

		if (! macro.isEmpty())
		{
			VymModel *m=currentModel();
			if (m) m->runScript(macro);
		}	
	}	
}

